/*
	File:		Binding.r

	Contains:	Alert dialog resources for Binding

	Owned by:	Reginald Adkins

	Copyright:	 1993 - 1996 by Apple Computer, Inc., all rights reserved.

	Change History (most recent first):

		 <5>	 6/27/96	jpa		1361886: Added NoPart error message table &
									alerts. Moved some rsrcs to new NoPart.res.
		 <4>	 6/27/96	RA		1361408: Added null viewer part handler
									viewer nmap
		 <3>	 3/15/96	RA		#1302213: New Editor Substituion dialog was
									misleading.
		 <2>	  1/8/96	CG		1299331 1.0.1: Replaced DITL with STR for
									change to notification manager.

	To Do:
		
*/


#define SystemSevenOrBetter 1			// we want the extended types
#define	SystemSevenOrLater	1			// Types.r uses this variable

#ifndef __TYPES_R__
#include "Types.r"
#endif

#ifndef __ODTYPES_R__
#include "ODTypes.r"
#endif

#ifndef SOM_Module_OpenDoc_Errors_defined
#include "ErrorDef.r"
#endif

#ifndef _CONSTDEF_
#include "ConstDef.h"
#endif

#ifndef _BNDNGDEF_
#include "BndngDef.h"
#endif


//==============================================================================
// Editor Substitution Warning
//==============================================================================


resource 'STR ' (kBindingWarning, purgeable)
{
	"One of the editors that was used to crea"
	"te some of the ^0 parts in this docume"
	"nt is missing.  ^1 has been substitute"
	"d."
};


//==============================================================================
// Fictitious Null Viewer 'nmap'
//==============================================================================


resource kODNameMappings (kNullViewerPartHandlerNMAP) 
{
	kODViewer,
	{	/* array Types: 1 elements */
		/* [1] */
		kNullViewerPartHandler,
		kODIsAnISOString
		{
			kODViewer
		}
	}
};


//==============================================================================
// NoPart Resources
//==============================================================================


include "NoPart.res" not 'ckid';
// Icon suite kNoPartIconResID
// Icon suite kNoPartButtonIconResID
// PICT		  kNoPartThumbnail


resource 'DLOG' (kNoPartDlogID, purgeable) {
	{80, 80, 258, 462},
	dBoxProc,
	invisible,
	noGoAway,
	0,//refcon
	kNoPartDlogID,
	"",
	alertPositionParentWindowScreen
};

resource 'DITL' (kNoPartDlogID, purgeable) {
	{
		/* [1] */
		{140, 279, 160, 353},
		Button {
			enabled,
			"OK"
		},
		/* [2] */
		{140, 175, 160, 259},
		Button {
			enabled,
			"Translate"
		},
		/* [3] */
		{10, 70, 135, 362},
		StaticText {
			disabled,
			"This gray box represents a part that could not be loaded.\n^0^2"
		},
		/* [4] */
		{141, 70, 159, 127},
		StaticText {
			disabled,
			"^3"
		},
		/* [5] */
		{10, 20, 42, 52},
		Icon {
			disabled,
			2
		}
	}
};


resource 'DLOG' (kNoPartRootDlogID, purgeable) {
	{80, 80, 258, 462},
	dBoxProc,
	invisible,
	noGoAway,
	0,//refcon
	kNoPartRootDlogID,
	"",
	alertPositionParentWindowScreen
};

resource 'DITL' (kNoPartRootDlogID, purgeable) {
	{
		/* [1] */
		{140, 279, 160, 353},
		Button {
			enabled,
			"OK"
		},
		/* [2] */
		{140, 175, 160, 259},
		Button {
			enabled,
			"Translate"
		},
		/* [3] */
		{10, 70, 135, 362},
		StaticText {
			disabled,
			"The document ^1 could not be opened. ^0^2"
		},
		/* [4] */
		{141, 70, 159, 127},
		StaticText {
			disabled,
			"^3"
		},
		/* [5] */
		{10, 20, 42, 52},
		Icon {
			disabled,
			0
		}
	}
};


//==============================================================================
// NoPart Error Message Table
//==============================================================================


type 'errs' {
	wide array {								// Error list
		LONGINT	 	whichList = 0,
					minErr = -2147483648;		// Low error number; 0 specifies STR#
		LONGINT		maxErr = 2147483647; 		// High error number
		LONGINT;								// Index in STR# of string
	};
};


resource 'errs' (kNoPartMessageTableID,purgeable) {
	{	whichList, 0, kNoPartMessageTableID;
	
		-1,					-1,					2;			// -1 denotes can't-find-editor (see NoPart.cpp)
		-2,					-2,					3;			// -2 is same but if we know the editor name
		-108,				-108,				4;
		kODErrOutOfMemory,	kODErrOutOfMemory,	4;
		-2802,				-2802,				5;			// fragSymbolNotFound
		-2804,				-2804,				6;			// fragLibNotFound
		-2807,				-2807,				5;			// fragHadUnresolveds
		-2810,				-2809,				4;			// fragNoAddrSpace..fragNoMem
		-2823,				-2823,				7;			// fragArchError
		
		-2899,				-2800,				8;			// all CFM errors
		
//		minErr,				maxErr,				1;			// all errors
	};
};


resource 'STR#' (kNoPartMessageTableID,purgeable) {
	{
		/*1*/"Could not load this ^1 part: ^2";				// Generic; loaded directly
		/*2*/"No editor could be found to edit this parts content.";
		/*3*/"The editor ^1 is not installed, and no installed editor can edit this parts content.";
		/*4*/"There was not enough memory available to load this part. Try closing other applications or documents.";
		/*5*/"Could not load the editor ^1 due to a missing symbol ^2. Try re-installing the editor.";
		/*6*/"Could not load the editor ^1 due to a missing library ^2. Try re-installing the editor.";
		/*7*/"The editor ^1 cannot run on this type of CPU. Try re-installing the editor.";
		/*8*/"Could not load the editor ^1 due to a shared-library error of type ^3. Try re-installing it.";
	};
};


resource 'STR#' (kNoPartTranslationStrID,purgeable) {
	{
		"\nNo translations are available for this data.",
		"The data can, however, be translated into a format readable by an installed editor."
	};
};
