/*
	File:		Core.r

	Contains:	nmap resources for OpenDoc

	Owned by:	Craig Carper

	Copyright:	 1993 - 1996 by Apple Computer, Inc., all rights reserved.

	Change History (most recent first):

		 <7>	 6/25/96	EL		1210509: defer to a later version.
		 <6>	 6/24/96	EL		1210509, 1218948: check in the missing
									alerts.
		 <5>	 6/22/96	EL		1210509, 1218948: alert for nmap conflict
									and delete pref file.
		 <4>	 6/21/96	RA		1358519: Added new category Archive
		 <3>	  6/4/96	EL		1283131: Separate Alerts for bad alias in
									editor folder and in library folder.
		 <2>	 5/17/96	EL		#1263100: Add progress bar and stop button
									to updating editor dialog.
		<16>	  9/7/95	CG		#1266854 FB:Bad Editor-alias warning.
		<15>	 8/30/95	CG		1257569 FB3:Added STR for OpenDoc prefs
									folder name.
		<14>	  8/8/95	jpa		Fixed layout/text of trashed-bad-alias
									alert. [1257556]
		<13>	 6/22/95	jpa		Added trashed-bad-alias alert [1257556]
		<12>	  6/8/95	RR		#1257260 Collapse base classes. #1214898
									StdTypes.r -> ODTypes.r
		<11>	 5/16/95	jpa		Added dialog kODScanEditorsDialogID. This
									is temporary. [1240150]
		<10>	 5/10/95	CG		1243604	The constant kNameMappings renamed
									to kODNameMappings.
		 <9>	 4/26/95	TJ		Removed version Resources, they are now in
									ODVersion.r
		 <8>	 3/29/95	NP		1233490: Use Mac system constant for
									international text stuff.
		 <7>	 3/21/95	T		1229970 Categories List needs to be updated
		 <6>	 2/16/95	T		#1220895 Core.r needs CategoryUserString
									nmap for Standard Categories.
		 <5>	  2/2/95	CG		#1207993, #1211081: Moved hard coded
									strings to resources.
		 <4>	 1/25/95	CC		1153802 Removed user string for 'styl'
									scrap type.
		 <3>	12/25/94	T		1191189 Eliminate duplicate Def.h, Def.xh
									files
		 <2>	 7/29/94	CG		Added vers 1 and 2 for use by prefs.
		 <1>	 7/11/94	T		first checked in
		 <0>	 7/11/94	SV		SOMverted
		 <2>	 3/18/94	CC		Include StdTypes.r to fix broken checkin.
									(1151640)
		 <1>	 3/18/94	CC		first checked in
		 <0>	 3/18/94	CC		Add user visible strings for Apple
		 							scrap types.  (1151640)

	To Do:
	In Progress:
		
*/


#define SystemSevenOrBetter 1			// we want the extended types
#define	SystemSevenOrLater	1			// Types.r uses this variable

#ifndef __ODTYPES_R__
#include "ODTypes.r"
#endif

#define oldTemp		//$$$$$ Needed to use old-style 'dctb' resource
#include "Types.r"
#undef oldTemp		//$$$$$ End of 'dctb' fix

#ifndef _CONSTDEF_
#include "ConstDef.h"
#endif

#ifndef _PREFSDEF_
#include "PrefsDef.h"
#endif

resource 'STR ' (kODPrefsFileNameStrID, "Prefs file name") {
	"OpenDoc Preferences"
};

resource 'STR ' (kODEditorsFldrNameStrID, "Editors Folder name") {
	"Editors"
};

resource 'STR ' (kODLibsFldrNameStrID, "OD Libraries Folder name") {
	"OpenDoc Libraries"
};

resource 'STR ' (kODPrefsFolderNameStrID, "Prefs folder name") {
	"OpenDoc Prefs"
};


// Dialog to display while scanning Editors/Libraries folders.
// This is here as a semi-hack to tell when this happens and how long it takes.
// I'll leave it up to the HI team to decide whether it can be allowed to stay. --jpa
resource 'DLOG' (kODScanEditorsDialogID) {
	{91, 92, 164, 368},
	noGrowDocProc,
	visible,
	noGoAway,
	0x0,
	kODScanEditorsDialogID,
	"",
	alertPositionParentWindowScreen
};

resource 'DITL' (kODScanEditorsDialogID) {
	{	/* array DITLarray: 3 elements */
		/* [1] */
		{46, 10, 57, 198},
		Picture {
			disabled,
			kScanEditorsProgressBarID
		},
		/* [2] */
		{10, 10, 28, 264},
		StaticText {
			disabled,
			"Updating OpenDoc editors database."
		},
		/* [3] */
		{42, 208, 62, 266},
		Button {
			enabled,
			"Stop"
		}
	}
};

resource 'dctb' (kODScanEditorsDialogID) {
	0x0,
	0,
	{	/* array ColorSpec: 5 elements */
		/* [1] */
		wContentColor, 65535, 65535, 65535,
		/* [2] */
		wFrameColor, 0, 0, 0,
		/* [3] */
		wTextColor, 0, 0, 0,
		/* [4] */
		wHiliteColor, 0, 0, 0,
		/* [5] */
		wTitleBarColor, 65535, 65535, 65535
	}
};

data 'ictb' (kODScanEditorsDialogID) {
	$"0000 0000 0000 0000 0000 0000"                      /* ............ */
};

data 'DLGX' (kODScanEditorsDialogID) {
	$"0743 6869 6361 676F 0000 0000 0000 0000"            /* .Chicago........ */
	$"0000 0000 0000 0000 0000 0000 0000 0000"            /* ................ */
	$"0000 0000 0000 0000 0000 0000 0000 0000"            /* ................ */
	$"0000 0000 0000 0000 0000 0000 0000 0000"            /* ................ */
	$"000C 0000 0000 0000 0001 0001 0000 0000"            /* ................ */
	$"0003 0009 0000 0000 0000 0000 0000 0006"            /* ............... */
	$"0000 0000 0000 0000 0000 0002 0000 0000"            /* ................ */
	$"0000 0000 0000"                                     /* ...... */
};

resource kPICTRsrcType (kScanEditorsProgressBarID, "pole #1", purgeable) {
	610,
	{0, 0, 11, 188},
	$"0011 02FF 0C00 FFFF FFFF 0000 0000 0000"
	$"0000 00BC 0000 000B 0000 0000 0000 0001"
	$"000A 0000 0000 000B 00BC 0098 8030 0000"
	$"0000 000B 00BC 0000 0000 0000 0000 0048"
	$"0000 0048 0000 0000 0002 0001 0002 0000"
	$"0000 00E5 4494 0000 0000 0000 04AD 0000"
	$"0002 0000 0000 0000 0000 0001 4444 4444"
	$"4444 0002 CCCC CCCC FFFF 0000 0000 000B"
	$"00BC 0000 0000 000B 00BC 0000 02D1 0031"
	$"2F15 556A AA95 556A AA95 556A AA95 556A"
	$"AA95 556A AA95 556A AA95 556A AA95 556A"
	$"AA95 556A AA95 556A AA95 556A AA95 5568"
	$"3031 2F25 555A AAA5 555A AAA5 555A AAA5"
	$"555A AAA5 555A AAA5 555A AAA5 555A AAA5"
	$"555A AAA5 555A AAA5 555A AAA5 555A AAA5"
	$"5558 0031 2F29 5556 AAA9 5556 AAA9 5556"
	$"AAA9 5556 AAA9 5556 AAA9 5556 AAA9 5556"
	$"AAA9 5556 AAA9 5556 AAA9 5556 AAA9 5556"
	$"AAA9 5554 6231 2F2A 5555 AAAA 5555 AAAA"
	$"5555 AAAA 5555 AAAA 5555 AAAA 5555 AAAA"
	$"5555 AAAA 5555 AAAA 5555 AAAA 5555 AAAA"
	$"5555 AAAA 5554 5031 2F2A 9555 6AAA 9555"
	$"6AAA 9555 6AAA 9555 6AAA 9555 6AAA 9555"
	$"6AAA 9555 6AAA 9555 6AAA 9555 6AAA 9555"
	$"6AAA 9555 6AAA 9554 0031 2F2A A555 5AAA"
	$"A555 5AAA A555 5AAA A555 5AAA A555 5AAA"
	$"A555 5AAA A555 5AAA A555 5AAA A555 5AAA"
	$"A555 5AAA A555 5AAA A554 0031 2F2A A955"
	$"56AA A955 56AA A955 56AA A955 56AA A955"
	$"56AA A955 56AA A955 56AA A955 56AA A955"
	$"56AA A955 56AA A955 56AA A954 0031 2F2A"
	$"AA55 55AA AA55 55AA AA55 55AA AA55 55AA"
	$"AA55 55AA AA55 55AA AA55 55AA AA55 55AA"
	$"AA55 55AA AA55 55AA AA55 55AA AA54 0031"
	$"2F2A AA95 556A AA95 556A AA95 556A AA95"
	$"556A AA95 556A AA95 556A AA95 556A AA95"
	$"556A AA95 556A AA95 556A AA95 556A AA94"
	$"4804 D200 0001 00FF"
};

resource kPICTRsrcType (kScanEditorsProgressBarID+1, "pole #2", purgeable) {
	612,
	{0, 0, 11, 188},
	$"0011 02FF 0C00 FFFF FFFF 0000 0000 0000"
	$"0000 00BC 0000 000B 0000 0000 0000 0001"
	$"000A 0000 0000 000B 00BC 0098 8030 0000"
	$"0000 000B 00BC 0000 0000 0000 0000 0048"
	$"0000 0048 0000 0000 0002 0001 0002 0000"
	$"0000 00E5 4258 0000 0000 0000 04C6 0000"
	$"0002 0000 0000 0000 0000 0001 CCCC CCCC"
	$"FFFF 0002 4444 4444 4444 0000 0000 000B"
	$"00BC 0000 0000 000B 00BC 0000 04D2 0000"
	$"CE31 2F15 6AAA 9555 6AAA 9555 6AAA 9555"
	$"6AAA 9555 6AAA 9555 6AAA 9555 6AAA 9555"
	$"6AAA 9555 6AAA 9555 6AAA 9555 6AAA 9555"
	$"6AA8 C731 2F15 5AAA A555 5AAA A555 5AAA"
	$"A555 5AAA A555 5AAA A555 5AAA A555 5AAA"
	$"A555 5AAA A555 5AAA A555 5AAA A555 5AAA"
	$"A555 5AA8 C531 2F15 56AA A955 56AA A955"
	$"56AA A955 56AA A955 56AA A955 56AA A955"
	$"56AA A955 56AA A955 56AA A955 56AA A955"
	$"56AA A955 56A8 BD31 2F15 55AA AA55 55AA"
	$"AA55 55AA AA55 55AA AA55 55AA AA55 55AA"
	$"AA55 55AA AA55 55AA AA55 55AA AA55 55AA"
	$"AA55 55AA AA55 55A8 B631 2F15 556A AA95"
	$"556A AA95 556A AA95 556A AA95 556A AA95"
	$"556A AA95 556A AA95 556A AA95 556A AA95"
	$"556A AA95 556A AA95 5568 CD31 2F25 555A"
	$"AAA5 555A AAA5 555A AAA5 555A AAA5 555A"
	$"AAA5 555A AAA5 555A AAA5 555A AAA5 555A"
	$"AAA5 555A AAA5 555A AAA5 5558 CB31 2F29"
	$"5556 AAA9 5556 AAA9 5556 AAA9 5556 AAA9"
	$"5556 AAA9 5556 AAA9 5556 AAA9 5556 AAA9"
	$"5556 AAA9 5556 AAA9 5556 AAA9 5554 C331"
	$"2F2A 5555 AAAA 5555 AAAA 5555 AAAA 5555"
	$"AAAA 5555 AAAA 5555 AAAA 5555 AAAA 5555"
	$"AAAA 5555 AAAA 5555 AAAA 5555 AAAA 5554"
	$"BC31 2F2A 9555 6AAA 9555 6AAA 9555 6AAA"
	$"9555 6AAA 9555 6AAA 9555 6AAA 9555 6AAA"
	$"9555 6AAA 9555 6AAA 9555 6AAA 9555 6AAA"
	$"9554 B504 D200 00AD 00FF"
};

resource kPICTRsrcType (kScanEditorsProgressBarID+2, "pole #3", purgeable) {
	612,
	{0, 0, 11, 188},
	$"0011 02FF 0C00 FFFF FFFF 0000 0000 0000"
	$"0000 00BC 0000 000B 0000 0000 0000 0001"
	$"000A 0000 0000 000B 00BC 0098 8030 0000"
	$"0000 000B 00BC 0000 0000 0000 0000 0048"
	$"0000 0048 0000 0000 0002 0001 0002 0000"
	$"0000 00E5 46F8 0000 0000 0000 04D2 0000"
	$"0002 0000 0000 0000 0000 0001 CCCC CCCC"
	$"FFFF 0002 4444 4444 4444 0000 0000 000B"
	$"00BC 0000 0000 000B 00BC 0000 04D2 0000"
	$"CE31 2F15 556A AA95 556A AA95 556A AA95"
	$"556A AA95 556A AA95 556A AA95 556A AA95"
	$"556A AA95 556A AA95 556A AA95 556A AA95"
	$"5568 C731 2F25 555A AAA5 555A AAA5 555A"
	$"AAA5 555A AAA5 555A AAA5 555A AAA5 555A"
	$"AAA5 555A AAA5 555A AAA5 555A AAA5 555A"
	$"AAA5 5558 C531 2F29 5556 AAA9 5556 AAA9"
	$"5556 AAA9 5556 AAA9 5556 AAA9 5556 AAA9"
	$"5556 AAA9 5556 AAA9 5556 AAA9 5556 AAA9"
	$"5556 AAA9 5554 BD31 2F2A 5555 AAAA 5555"
	$"AAAA 5555 AAAA 5555 AAAA 5555 AAAA 5555"
	$"AAAA 5555 AAAA 5555 AAAA 5555 AAAA 5555"
	$"AAAA 5555 AAAA 5554 B631 2F2A 9555 6AAA"
	$"9555 6AAA 9555 6AAA 9555 6AAA 9555 6AAA"
	$"9555 6AAA 9555 6AAA 9555 6AAA 9555 6AAA"
	$"9555 6AAA 9555 6AAA 9554 CD31 2F2A A555"
	$"5AAA A555 5AAA A555 5AAA A555 5AAA A555"
	$"5AAA A555 5AAA A555 5AAA A555 5AAA A555"
	$"5AAA A555 5AAA A555 5AAA A554 CB31 2F2A"
	$"A955 56AA A955 56AA A955 56AA A955 56AA"
	$"A955 56AA A955 56AA A955 56AA A955 56AA"
	$"A955 56AA A955 56AA A955 56AA A954 C331"
	$"2F2A AA55 55AA AA55 55AA AA55 55AA AA55"
	$"55AA AA55 55AA AA55 55AA AA55 55AA AA55"
	$"55AA AA55 55AA AA55 55AA AA55 55AA AA54"
	$"BC31 2F2A AA95 556A AA95 556A AA95 556A"
	$"AA95 556A AA95 556A AA95 556A AA95 556A"
	$"AA95 556A AA95 556A AA95 556A AA95 556A"
	$"AA94 B504 D200 00AD 00FF"
};

resource kPICTRsrcType (kScanEditorsProgressBarID+3, "pole #4", purgeable) {
	612,
	{0, 0, 11, 188},
	$"0011 02FF 0C00 FFFF FFFF 0000 0000 0000"
	$"0000 00BC 0000 000B 0000 0000 0000 0001"
	$"000A 0000 0000 000B 00BC 0098 8030 0000"
	$"0000 000B 00BC 0000 0000 0000 0000 0048"
	$"0000 0048 0000 0000 0002 0001 0002 0000"
	$"0000 00E5 4168 0000 0000 0000 04D7 0000"
	$"0002 0000 0000 0000 0000 0001 4444 4444"
	$"4444 0002 CCCC CCCC FFFF 0000 0000 000B"
	$"00BC 0000 0000 000B 00BC 0000 04D2 0000"
	$"CE31 2F15 6AAA 9555 6AAA 9555 6AAA 9555"
	$"6AAA 9555 6AAA 9555 6AAA 9555 6AAA 9555"
	$"6AAA 9555 6AAA 9555 6AAA 9555 6AAA 9555"
	$"6AA8 C731 2F15 5AAA A555 5AAA A555 5AAA"
	$"A555 5AAA A555 5AAA A555 5AAA A555 5AAA"
	$"A555 5AAA A555 5AAA A555 5AAA A555 5AAA"
	$"A555 5AA8 C531 2F15 56AA A955 56AA A955"
	$"56AA A955 56AA A955 56AA A955 56AA A955"
	$"56AA A955 56AA A955 56AA A955 56AA A955"
	$"56AA A955 56A8 BD31 2F15 55AA AA55 55AA"
	$"AA55 55AA AA55 55AA AA55 55AA AA55 55AA"
	$"AA55 55AA AA55 55AA AA55 55AA AA55 55AA"
	$"AA55 55AA AA55 55A8 B631 2F15 556A AA95"
	$"556A AA95 556A AA95 556A AA95 556A AA95"
	$"556A AA95 556A AA95 556A AA95 556A AA95"
	$"556A AA95 556A AA95 5568 CD31 2F25 555A"
	$"AAA5 555A AAA5 555A AAA5 555A AAA5 555A"
	$"AAA5 555A AAA5 555A AAA5 555A AAA5 555A"
	$"AAA5 555A AAA5 555A AAA5 5558 CB31 2F29"
	$"5556 AAA9 5556 AAA9 5556 AAA9 5556 AAA9"
	$"5556 AAA9 5556 AAA9 5556 AAA9 5556 AAA9"
	$"5556 AAA9 5556 AAA9 5556 AAA9 5554 C331"
	$"2F2A 5555 AAAA 5555 AAAA 5555 AAAA 5555"
	$"AAAA 5555 AAAA 5555 AAAA 5555 AAAA 5555"
	$"AAAA 5555 AAAA 5555 AAAA 5555 AAAA 5554"
	$"BC31 2F2A 9555 6AAA 9555 6AAA 9555 6AAA"
	$"9555 6AAA 9555 6AAA 9555 6AAA 9555 6AAA"
	$"9555 6AAA 9555 6AAA 9555 6AAA 9555 6AAA"
	$"9554 B504 D200 00AD 00FF"
};

resource 'ALRT' (kODBadAliasInEditorsAlertID) {
	{44, 25, 173, 441},
	kODBadAliasInEditorsAlertID,
	{	/* array: 4 elements */
		/* [1] */
		OK, visible, sound1,
		/* [2] */
		OK, visible, sound1,
		/* [3] */
		OK, visible, sound1,
		/* [4] */
		OK, visible, sound1
	}
#if SystemSevenOrLater
	,
	centerMainScreen
#endif
};

resource 'DITL' (kODBadAliasInEditorsAlertID) {
	{	/* array DITLarray: 3 elements */
		/* [1] */
		{99, 342, 119, 400},
		Button {
			enabled,
			"OK"
		},
		/* [2] */
		{10, 62, 90, 400},
		StaticText {
			disabled,
			"The Editors folder can't contain aliases"
			" to items on other volumes. Aliases to e"
			"ditors on other volumes were found and w"
			"ere moved to the Trash. To use aliases, "
			"make sure the original items are on the "
			"same volume as the Editors folder."
		},
		/* [3] */
		{10, 20, 42, 52},
		Icon {
			disabled,
			1
		}
	}
};

resource 'ALRT' (kODBadAliasInLibrariesAlertID) {
	{44, 25, 173, 441},
	kODBadAliasInLibrariesAlertID,
	{	/* array: 4 elements */
		/* [1] */
		OK, visible, sound1,
		/* [2] */
		OK, visible, sound1,
		/* [3] */
		OK, visible, sound1,
		/* [4] */
		OK, visible, sound1
	}
#if SystemSevenOrLater
	,
	centerMainScreen
#endif
};

resource 'DITL' (kODBadAliasInLibrariesAlertID) {
	{	/* array DITLarray: 3 elements */
		/* [1] */
		{99, 342, 119, 400},
		Button {
			enabled,
			"OK"
		},
		/* [2] */
		{10, 62, 90, 400},
		StaticText {
			disabled,
			"The OpenDoc Libraries folder can't conta"
			"in aliases to items on other volumes. Su"
			"ch aliases were found and were moved to "
			"the Trash. To use aliases, make sure the"
			" original items are on the same volume a"
			"s the OpenDoc Libraries folder."
		},
		/* [3] */
		{10, 20, 42, 52},
		Icon {
			disabled,
			1
		}
	}
};

//resource 'ALRT' (kODEditorNmapClashAlertID, "Editor nmap clash") {
//	{40, 40, 173, 415},
//	kODEditorNmapClashAlertID,
//	{	/* array: 4 elements */
//		/* [1] */
//		OK, visible, sound1,
//		/* [2] */
//		OK, visible, sound1,
//		/* [3] */
//		OK, visible, sound1,
//		/* [4] */
//		OK, visible, sound1
//	}
//#if SystemSevenOrLater
//	,
//	centerMainScreen
//#endif
//};
//
//resource 'DITL' (kODEditorNmapClashAlertID, "Editor nmap clash") {
//	{	/* array DITLarray: 4 elements */
//		/* [1] */
//		{99, 301, 119, 359},
//		Button {
//			enabled,
//			"OK"
//		},
//		/* [2] */
//		{7, 65, 87, 359},
//		StaticText {
//			disabled,
//			"The editor ^0 conflicts with ^1.  Yo"
//			"u can only use one of them.  Please remo"
//			"ve the one you don't want to use from yo"
//			"ur Editors folder and try  again."
//		},
//		/* [3] */
//		{10, 20, 42, 52},
//		Icon {
//			disabled,
//			1
//		},
//		/* [4] */
//		{95, 297, 123, 363},
//		UserItem {
//			disabled
//		}
//	}
//};

resource 'ALRT' (kODEditorPrefsDamagedAlertID, "Editor prefs damaged") {
	{40, 40, 141, 359},
	kODEditorPrefsDamagedAlertID,
	{	/* array: 4 elements */
		/* [1] */
		OK, visible, sound1,
		/* [2] */
		OK, visible, sound1,
		/* [3] */
		OK, visible, sound1,
		/* [4] */
		OK, visible, sound1
	}
#if SystemSevenOrLater
	,
	centerMainScreen
#endif
};

resource 'DITL' (kODEditorPrefsDamagedAlertID, "Editor prefs damaged") {
	{	/* array DITLarray: 4 elements */
		/* [1] */
		{71, 246, 91, 304},
		Button {
			enabled,
			"OK"
		},
		/* [2] */
		{10, 65, 58, 302},
		StaticText {
			disabled,
			"The OpenDoc editor preferences are damag"
			"ed. Please reset them using the Editor "
			"Setup control panel."
		},
		/* [3] */
		{10, 20, 42, 52},
		Icon {
			disabled,
			1
		},
		/* [4] */
		{67, 242, 95, 308},
		UserItem {
			disabled
		}
	}
};

// Standard Category 'nmap' and their user strings.

resource kODNameMappings (301) {
	kODCategoryUserString,
	{	/* array KeyList: Many elements */
		/* [1] */
		kODCategoryPlainText,
		kODIsINTLText {
			smRoman,
			langEnglish,
			"Plain Text"
		},
		
		/* [2] */
		kODCategoryStyledText,
		kODIsINTLText {
			smRoman,
			langEnglish,
			"Styled Text"
		},

		/* [3] */
		kODCategoryDrawing,
		kODIsINTLText {
			smRoman,
			langEnglish,
			"Drawing"
		},

		/* [4] */
		kODCategory3DGraphic,
		kODIsINTLText {
			smRoman,
			langEnglish,
			"3D Graphic"
		},

		/* [5] */
		kODCategoryPainting,
		kODIsINTLText {
			smRoman,
			langEnglish,
			"Painting"
		},

		/* [6] */
		kODCategoryMovie,
		kODIsINTLText {
			smRoman,
			langEnglish,
			"Movie"
		},

		/* [7] */
		kODCategorySampledSound,
		kODIsINTLText {
			smRoman,
			langEnglish,
			"Sampled Sound"
		},

		/* [8] */
		kODCategoryChart,
		kODIsINTLText {
			smRoman,
			langEnglish,
			"Chart"
		},

		/* [9] */
		kODCategoryFormula,
		kODIsINTLText {
			smRoman,
			langEnglish,
			"Formula"
		},

		/* [10] */
		kODCategorySpreadsheet,
		kODIsINTLText {
			smRoman,
			langEnglish,
			"Spreadsheet"
		},

		/* [11] */
		kODCategoryTable,
		kODIsINTLText {
			smRoman,
			langEnglish,
			"Table"
		},

		/* [12] */
		kODCategoryDatabase,
		kODIsINTLText {
			smRoman,
			langEnglish,
			"Database"
		},

		/* [13] */
		kODCategoryQuery,
		kODIsINTLText {
			smRoman,
			langEnglish,
			"Query"
		},

		/* [14] */
		kODCategoryConnection,
		kODIsINTLText {
			smRoman,
			langEnglish,
			"Connection"
		},

		/* [15] */
		kODCategoryScript,
		kODIsINTLText {
			smRoman,
			langEnglish,
			"Script"
		},

		/* [16] */
		kODCategoryOutline,
		kODIsINTLText {
			smRoman,
			langEnglish,
			"Outline"
		},

		/* [17] */
		kODCategoryPageLayout,
		kODIsINTLText {
			smRoman,
			langEnglish,
			"Page Layout"
		},

		/* [18] */
		kODCategoryPresentation,
		kODIsINTLText {
			smRoman,
			langEnglish,
			"Presentation"
		},

		/* [19] */
		kODCategoryCalendar,
		kODIsINTLText {
			smRoman,
			langEnglish,
			"Calendar"
		},

		/* [20] */
		kODCategoryForm,
		kODIsINTLText {
			smRoman,
			langEnglish,
			"Form"
		},

		/* [21] */
		kODCategoryExecutable,
		kODIsINTLText {
			smRoman,
			langEnglish,
			"Executable"
		},

		/* [22] */
		kODCategoryCompressed,
		kODIsINTLText {
			smRoman,
			langEnglish,
			"Compressed"
		},

		/* [23] */
		kODCategoryControlPanel,
		kODIsINTLText {
			smRoman,
			langEnglish,
			"Control Panel"
		},

		/* [24] */
		kODCategoryControl,
		kODIsINTLText {
			smRoman,
			langEnglish,
			"Control"
		},

		/* [25] */
		kODCategoryPersonalInfo,
		kODIsINTLText {
			smRoman,
			langEnglish,
			"Personal Info"
		},

		/* [26] */
		kODCategorySpace,
		kODIsINTLText {
			smRoman,
			langEnglish,
			"Space"
		},

		/* [27] */
		kODCategoryProject,
		kODIsINTLText {
			smRoman,
			langEnglish,
			"Project"
		},

		/* [28] */
		kODCategorySignature,
		kODIsINTLText {
			smRoman,
			langEnglish,
			"Signature"
		},

		/* [29] */
		kODCategoryKey,
		kODIsINTLText {
			smRoman,
			langEnglish,
			"Key"
		},

		/* [30] */
		kODCategoryUtility,
		kODIsINTLText {
			smRoman,
			langEnglish,
			"Utility"
		},

		/* [31] */
		kODCategoryMailingLabel,
		kODIsINTLText {
			smRoman,
			langEnglish,
			"Mailing Label"
		},

		/* [32] */
		kODCategoryLocator,
		kODIsINTLText {
			smRoman,
			langEnglish,
			"Locator"
		},

		/* [33] */
		kODCategoryPrinter,
		kODIsINTLText {
			smRoman,
			langEnglish,
			"Printer"
		},

		/* [34] */
		kODCategoryStructuredSound,
		kODIsINTLText {
			smRoman,
			langEnglish,
			"Structured Sound"
		},

		/* [35] */
		kODCategoryTime,
		kODIsINTLText {
			smRoman,
			langEnglish,
			"Time"
		},
		
		/* [n] */
		kODCategoryArchive,
		kODIsINTLText {
			smRoman,
			langEnglish,
			"Archive"
		}
	}
};
