/*
	File:		InfoDefs.h

	Contains:	constants used in both the code & resource of Info

	Owned by:	Ed Lai

	Copyright:	 1993 - 1996 by Apple Computer, Inc., all rights reserved.

	Change History (most recent first):

		 <5>	 6/21/96	eeh		task 10008: add buttons etc. for AppleGuide
		 <4>	 6/19/96	EL		1358681: Resource IDs are in the reserved
									range.
		 <3>	  6/4/96	EL		1323543: Bundled checkbox is now two radio
									buttons.
		 <2>	 3/20/96	EL		1289563: define more constants for the heap
									size dialog
		<10>	10/16/95	CC		GM 1292157: Added kPartInfoEditorLabel and
									kPartInfoNoEditorText.
		 <9>	 4/27/95	T		1223529 BB: ODSetPOName and ODSetIconFamily
									should tweak file in root part case (Did
									name piece)
		 <8>	 3/28/95	T		#1151710 BB: Toolbox running out of app
									heap.  Document Size dialog implemented.
		 <7>	 3/24/95	CG		Brought back Editor ID field.
		 <6>	 3/23/95	T		#1151710 BB: Toolbox running out of app
									heap. (Not done).  Document Size dialog
									partially implemented.  Added constants for
									resources.
		 <5>	 2/24/95	CG		1221029, 1221185: Info dialog appearance
									problems.
		 <4>	 2/10/95	CG		#1187539: Renaming a document using
									Document Info handles file name conflicts.
		 <3>	 9/26/94	T		#1188044 Part Info Checkbox behaviour
		 <2>	 9/15/94	T		#1183571 implement Settings Extension
		 <1>	 7/11/94	T		first checked in
		 <1>	 7/11/94	T		first checked in
		 <1>	 7/11/94	T		first checked in
		 <0>	  7/5/94	SV		SOMverted
		 <2>	 4/13/94	T		#1147222: Fix the View As popup in the Part
									Frame Info dialog.  Also fix the Category
									text, and the Kind & Editor popups to show
									the current kind & editor respectively.
		 <1>	11/18/93	T		first checked in

	To Do:
*/

#ifndef _INFODEFS_
#define _INFODEFS_

#ifndef _RSRCBASE_
#include "RsrcBase.h"
#endif

#define kPartInfoDlgID        	(kODCoreRsrcBase+10)
#define kPartInfoReadOnlyPictID	(kODCoreRsrcBase+10)
#define	kPartInfoKindCntl		(kODCoreRsrcBase+11)
#define	kPartInfoKindMenu		(kODCoreRsrcBase+11)
#define	kPartInfoEditorCntl		(kODCoreRsrcBase+12)
#define	kPartInfoEditorMenu		(kODCoreRsrcBase+12)
#define	kPartInfoViewAsCntl		(kODCoreRsrcBase+13)
#define	kPartInfoViewAsMenu		(kODCoreRsrcBase+13)

#define kPartInfoInvalidNameAlertID	(kODCoreRsrcBase+12)
#define kPartInfoLongCommentAlertID	(kODCoreRsrcBase+13)


#define kPartInfoVertSep     		25
#define kPartInfoHorizSep			26

/* Changeable fields in Part Info dialog */
#define kPartInfoOKBtn		   		1
#define kPartInfoCancelBtn			2
#define kPartInfoSettingsButton	    3
#define kPartInfoCommentsEditFld    4
#define kPartInfoByFld				5
#define kPartInfoModifiedFld		7
#define kPartInfoCreatedFld			9
#define kPartInfoSizeFld			11
#define kPartInfoSep1     		    13
#define kPartInfoEditorPopup		14
#define kPartInfoKindPopup			15
#define kPartInfoCategoryFld		16
#define kPartInfoSep2			    18
#define kPartInfoShowLinksCheckbox  19
#define kPartInfoShowLinksText		20
#define kPartInfoStationeryCheckbox	21
#define kPartInfoStationeryText		22
#define kPartInfoActPartRadBtn    	23
#define kPartInfoSelPartRadBtn    	24
#define kPartInfoViewAsPopup		25
#define kPartInfoNameEditFld		26
#define kPartInfoPartThumbnail      28
#define kPartInfoDefaultButtonItem	29
#define kPartInfoIDFld				30
#define kPartInfoMemorySizeButton	32
#define kPartInfoEditorLabel		33
#define kPartInfoNoEditorText		34
#define kPartInfoBundledText		35
#define kPartInfoReadOnlyPict		36
#define kPartInfoReadOnlyText		37
#define kPartInfoActPartText    	38
#define kPartInfoSelPartText    	39
#define kPartInfoAppleGuideButton	40

// Strings used by Part Info dialogs
#define kODPartInfoSizeInBytes			(kODCoreRsrcBase+10)
#define kODPartInfoSizeInK				(kODCoreRsrcBase+11)
#define	kODDocumentInfoWindowName		(kODCoreRsrcBase+12)
#define kODPartInfoStrUnknownID			(kODUtilitiesRsrcBase+0)

// Other Strings used by InfoUtil
#define	kODInfoUtilStrsID				(kODUtilitiesRsrcBase+1)
#define kODStrIndNoKinds					1
#define kODStrIndNoEditors					2

/* Document Heap dialog defines */
/* resource ids in the 265-269 range in this file are reserved for the Doc Heap dialog */

#define kDocHeapDlgID        	(kODCoreRsrcBase+15)

#define kArrowBtnsCntlID		(kODCoreRsrcBase+15)

#define kPICTRsrcType			'PICT'
#define kSIZERsrcType			'SIZE'

#define kArrowBtnsPictOffID		(kODCoreRsrcBase+15)
#define kArrowBtnsPictTopOnID	(kODCoreRsrcBase+16)
#define kArrowBtnsPictBotOnID	(kODCoreRsrcBase+17)


#define kDocHeapOKBtn		   		1
#define kDocHeapCancelBtn			2
#define kDocHeapDefaultSizeRadBtn	3
#define kDocHeapDefaultSizeFld    	4
#define kDocHeapDefaultSizeBotBtnRect 5
#define kDocHeapPreferdSizeRadBtn	6
#define kDocHeapPreferdSizeFld    	7
#define kDocHeapPreferdSizeBotBtnRect 8
#define kDocHeapMakeChangeStatText 	9
#define kDocHeapDocumentNameFld		10
#define kDocHeapDefaultKStatText    11
#define kDocHeapPreferdKStatText    12
#define kDocHeapDefaultButtonItem	13
#define kDocHeapDefaultSizeTopBtnRect 14
#define kDocHeapDefaultSizeBtns    	15
#define kDocHeapPreferdSizeTopBtnRect 16
#define kDocHeapPreferdSizeBtns    	17
#define kDocHeapChangeNextLaunchStatText 	20


#endif // _INFODEFS_
