/*
	File:		ErrMsgs.r

	Contains:	Error Messages for the document shell

	Owned by:	Nick Pilch
 
	Copyright:	 1994 - 1996 by Apple Computer, Inc., all rights reserved.

	Change History (most recent first):
        
		 <4>	 6/18/96	NP		1354567: No error message
									kODErrInvalidPermissions.
		 <3>	.04.1996	NP		1302783, 1320564, 1333505: misspellings in
									error dialog.
		 <2>	 3/22/96	CC		1330388: Added kODErrCannotDeferClone
		<28>	30/10/95	NP		1295276,1297278,1297281: Problems with
									reporting private errors.
		<27>	10/24/95	jpa		1293441: Added kODErrFatalContainerError
		<26>	23/10/95	NP		1293899, 1293026: Error messages and
									mapping tweaks.
		<25>	17/10/95	NP		1292926: Duplicate entry 79 in STR#
									resource.
		<24>	12/10/95	NP		1284573: Final mods to error messages and
									mappings.
		<23>	 10/9/95	VL		1288571: Added message for
									kODErrNoSysTranslationFacility.
		<22>	 10/4/95	NP		1282801, 1285707, 1287830: Error
									codes/messages changes.
		<21>	 9/29/95	eeh		1287262: use ShellDef.h
		<20>	 9/18/95	CG		1285279 GM: kODPartNotWrapper should be
									kODErrPartNotWrapper.
		<19>	 9/12/95	DM		1278291: kODErrInvalidName ->
									kODErrInvalidNSName
		<18>	 9/11/95	NP		Oops. Fixed file so it compiles.
		<17>	 9/11/95	NP		1281307: Need MacOS error mssgs. 1282801:
									Some error messages missing (partial).
		<16>	  9/8/95	T		1252120 FB1: New documents saved w/ StdFile
									& trashed if closed w/o saving
		<15>	  9/8/95	VL		1282012: Removed kODErrDraftNotExists and
									kODErrContainerNotExists.
		<14>	  9/8/95	jpa		Removed obsolete errors. Removed "invalid"
									from names of errors that are actually
									still valid. [1281354]
		<13>	  9/8/95	T		1263308 FB3:Cryptic error msg when making
									stationery from part not in the Editors
									folder
		<12>	 8/24/95	EL		1278212: kODErrNoDocumentProperties no
									longer needed.
		<11>	 8/21/95	NP		1274946:
									kODErrInvalidCanvas->kODErrCanvasNotFound
		<10>	 8/18/95	NP		1274946: Had to remove some stuff. Still
									need to fix up this file.
		 <9>	 8/16/95	NP		1274946: ErrorDef.idl problems.
		 <8>	  6/9/95	TJ		Include ErrorDef.r instead of ErrorDef.xh
		 <7>	  6/1/95	jpa		Added msg for kODErrCantLoadSOMClass. Fixed
									messages for kODErrCantCreatePart and
									kODErrCantCreateContainer. [1242839]
		 <6>	 5/25/95	VL		1251403: Multithreading naming support.
		 <5>	 5/18/95	CC		1250280: Track rename to
									kODErrNullFacetInput.
		 <4>	 4/14/95	T		#1221135	DR/BB: When OpenDoc Libraries
									folder is missing, OpenDoc reports generic
									error 84
									#1239535	DR/BB: When Editors folder is
									missing, OpenDoc reports undefined error
									173.
		 <3>	 3/28/95	T		#1196779 BB: Better error message for
									Saving a Copy onto oneself
		 <2+>	 3/23/95	MS		Added "invalid error code" message strings for the 
		 							following codes marked $$$$$:  kODErrAssertionFailed,
									kODErrInvalidParameter, kODErrInvalidFrame, 
									kODErrInvalidFacet, kODErrInvalidValue, 
									kODErrInvalidRefCount, kODErrInvalidGraphicsSystem, 
									kODErrTransformErr, kODErrInvalidCanvas, 
									kODErrMemoryProblem, kODErrInvalidNSName, 
									kODErrInvalidContainer, kODErrInvalidDocument, 
									kODErrInvalidDraft, kODErrInvalidStorageUnit, 
									kODErrInvalidType, kODErrInvalidProperty, 
									kODErrInvalidID, kODErrInconsistentCloneKind, 
									kODErrInvalidLinkKey, kODErrCannotCreateContainer, 
									and kODErrCannotOpenContainer. Also, changed string 
									reference for kOKErrWriteErr and kODErrZeroRefCount.
		 		 3/22/95	MS		Added User ID message strings and Support ID 
				 					placeholders for the following new error codes: 
									kODErrPartInUse, 
									kODErrCouldNotBindForeign Document, 
									kODErrSystemVolumeIsFull, kODErrRefCountNotEqualOne, 
									kODErrAllWritableVolumesAreFull, 
									kODErrDirectoryNotHaveWriteAccess, 
									kODErrCannotSaveACopyOntoCurrentDoc, 
									kODErrCannotSaveACopyOntoBusyDoc, kODErrFileClosed, 
									kODErrFileOpened, kODErrSOMException, 
									kODErrCorruptLinkSource, kODErrCorruptLink, 
									kODErrCannotRevealLink, 
									kODErrCannotFindLinkSourceEdition, 
									kODErrCannotFindLinkSource, kODErrAlreadyImportedLink,
									kODErrLinkAlreadyExported, kODErrInvalidObjectType, 
									kODErrInvalidPersistentObjectID, kODErrNullFacet, 
									and kODErrIllegalNullStorageSystemInput.  
		 <2>	 3/21/95	T		Tweaked a couple of messages.
		 <1>	12/25/94	T		first checked in
		 <0.4>	12/25/94	T		1191189 Eliminate duplicate Def.h, Def.xh files
									1187709 More frequent, robust and earlier error 
									detection and reporting in the Shell.
									Fixed resources so that they would compile with Rez.

         <0.3>	12/12/94    MS		entered more UserID error codes
         <0.2>	 12/5/94    MS		entered a few UserID errors for format check by Tantek
         <0.1>	12/19/94    MS		alpha draft for UserID and SupportID error codes and 
		 							strings

	Note(s) from Tantek:
		1. Make sure to NOT put in things like
			 <"name">
		   in the middle of strings.  It doesn't work because quotes do not nest in Rez.
		2. Try to make sure that Change History comments are no more than 80 chars 
		   per line.
		3. DO NOT DELETE ANY STRINGS FROM THE STR# resources. Simply empty them out.
		   Like this:   ;""
		   This we they don't get renumbered, we don't have to try and fix up the
		   'errs' resources to point to the renumbered indexes into the STR# resources.
		
	In Progress:
		
*/

#define SystemSevenOrBetter 1			// we want the extended types
#define	SystemSevenOrLater	1			// Types.r uses this variable

#define oldTemp			// to get 'dctb' resources to rez
#ifndef __TYPES_R__
#include "Types.r"
#endif
#undef oldTemp

#ifndef _SHELLDEF_
#include "ShellDef.h"
#endif

#ifndef SOM_Module_OpenDoc_Errors_defined
#include "ErrorDef.r"
#endif

/* list mapping exception codes to user explanations */

/* See ErrorDef.idl for range information */

#define minUtilsErr -29849
#define maxUtilsErr -29830
#define minUIErr -29829
#define maxUIErr -29827
#define minShellErr -29826
#define maxShellErr -29810
#define minODBentoErr -29809
#define maxODBentoErr -29789
#define minBentoErr 1000
#define maxBentoErr 1999

/*-------------------------------------errs-------------------------------------*/
type 'errs' {
	wide array {								// Error list
		LONGINT	 	whichList = 0,
					minErr = -2147483648;		// Low error number; 0 specifies STR#
		LONGINT		maxErr = 2147483647; 		// High error number
		LONGINT;								// Index in STR# of string
	};
};

resource 'errs' (kODErrUserID,purgeable) {
	{	whichList, 0, kODErrUserIDStrings
			;kODErrInvalidFrame, kODErrInvalidFrame, 1
			;kODErrInvalidFacet, kODErrInvalidFacet, 1
			;kODErrIllegalNullInput, kODErrIllegalNullInput, 1
			;kODErrObjectNotInitialized, kODErrObjectNotInitialized, 1
			;kODErrOutOfMemory, kODErrOutOfMemory, 3
			;kODErrNotImplemented, kODErrNotImplemented, 1

			;kODErrSubClassResponsibility, kODErrSubClassResponsibility, 1
			;kODErrUnsupportedExtension, kODErrUnsupportedExtension, 1
			;kODErrInvalidObjectType, kODErrInvalidObjectType, 1
			;kODErrInvalidPersistentObjectID, kODErrInvalidPersistentObjectID, 1
			;kODErrRefCountNotEqualOne, kODErrRefCountNotEqualOne, 1
			;kODErrZeroRefCount, kODErrZeroRefCount, 1
			;kODErrIteratorOutOfSync, kODErrIteratorOutOfSync, 1

			;kODErrCannotEmbed, kODErrCannotEmbed, 8
			;kODErrKeyAlreadyExists, kODErrKeyAlreadyExists, 1
			;kODErrPartInUse, kODErrPartInUse, 45

	/* DOCSHELL PRIVATE CODES */
			;kODErrDirectoryNotHaveWriteAccess, kODErrDirectoryNotHaveWriteAccess, 10
			;kODErrCouldNotBindForeignDocument, kODErrCouldNotBindForeignDocument, 33
			;kODErrAllWritableVolumesAreFull, kODErrAllWritableVolumesAreFull, 34
			;kODErrSystemVolumeIsFull, kODErrSystemVolumeIsFull, 35
			;kODErrCannotSaveACopyOntoCurrentDoc, kODErrCannotSaveACopyOntoCurrentDoc, 36
			;kODErrCannotSaveACopyOntoBusyDoc, kODErrCannotSaveACopyOntoBusyDoc, 37
			;kODErrCannotFindSystemEditorsFolder, kODErrCannotFindSystemEditorsFolder, 70
			;kODErrCannotFindOpenDocLibrariesFolder, kODErrCannotFindOpenDocLibrariesFolder, 71
			;kODErrEditorLibraryNotFound, kODErrEditorLibraryNotFound, 74
			;kODErrCannotSaveAsOntoBusyDoc, kODErrCannotSaveAsOntoBusyDoc, 75
	/* END DOCSHELL PRIVATE CODES */

			;kODErrInvalidGraphicsSystem, kODErrInvalidGraphicsSystem, 1
			;kODErrNoShapeGeometry, kODErrNoShapeGeometry, 1
			;kODErrTransformErr, kODErrTransformErr, 1
			;kODErrCanvasNotFound, kODErrCanvasNotFound, 1
			;kODErrUnsupportedFramePositionCode, kODErrUnsupportedFramePositionCode, 1
			;kODErrIllegalNullPartInput, kODErrIllegalNullPartInput, 1
			;kODErrIllegalNullTransformInput, kODErrIllegalNullTransformInput, 1
			;kODErrIllegalNullStorageSystemInput, kODErrIllegalNullStorageSystemInput, 1
			;kODErrInvalidBlock, kODErrInvalidBlock, 45

			;kODErrUnsupportedPosCode, kODErrUnsupportedPosCode, 1
			;kODErrInvalidPermissions, kODErrInvalidPermissions, 14
			;kODErrInvalidNSName, kODErrInvalidNSName, 1
			;kODErrInvalidStorageUnit, kODErrInvalidStorageUnit, 1
			;kODErrNoValueAtThatIndex, kODErrNoValueAtThatIndex, 1
			;kODErrCannotAddProperty, kODErrCannotAddProperty, 1

			;kODErrUnfocusedStorageUnit, kODErrUnfocusedStorageUnit, 1
			;kODErrIllegalNullPropertyInput, kODErrIllegalNullPropertyInput, 1
			;kODErrIllegalNullSUCursorInput, kODErrIllegalNullSUCursorInput, 1
			;kODErrInvalidStorageUnitRef, kODErrInvalidStorageUnitRef, 1
			;kODErrInvalidStorageUnitKey, kODErrInvalidStorageUnitKey, 1
			;kODErrStorageUnitNotLocked, kODErrStorageUnitNotLocked, 1
			;kODErrNoDraftProperties, kODErrNoDraftProperties, 1

			;kODErrCannotCreateFrame, kODErrCannotCreateFrame, 1
			;kODErrCannotAcquireFrame, kODErrCannotAcquireFrame, 1
			;kODErrCannotCreatePart, kODErrCannotCreatePart, 16
			;kODErrCannotAcquirePart, kODErrCannotAcquirePart, 1
			;kODErrCannotCreateLink, kODErrCannotCreateLink, 1
			;kODErrCannotAcquireLink, kODErrCannotAcquireLink, 1
			;kODErrInvalidID, kODErrInvalidID, 1
			;kODErrInconsistentCloneKind, kODErrInconsistentCloneKind, 1
			;kODErrInvalidCloneKind, kODErrInvalidCloneKind, 1
			;kODErrInvalidBelowDraft, kODErrInvalidBelowDraft, 1

			;kODErrCannotCollapseDrafts, kODErrCannotCollapseDrafts, 18
			;kODErrNonEmptyDraft, kODErrNonEmptyDraft, 1
			;kODErrNoPreviousDraft, kODErrNoPreviousDraft, 1
			;kODErrOutstandingDraft, kODErrOutstandingDraft, 1
			;kODErrInvalidDraftID, kODErrInvalidDraftID, 1
			;kODErrCannotChangePermissions, kODErrCannotChangePermissions, 1
			;kODErrContainerExists, kODErrContainerExists, 1
			;kODErrCannotGetExternalLink, kODErrCannotGetExternalLink, 21

			;kODErrNoLinkSpecValue, kODErrNoLinkSpecValue, 1
			;kODErrUnknownLinkSpecVersion, kODErrUnknownLinkSpecVersion, 1
			;kODErrCorruptLinkSpecValue, kODErrCorruptLinkSpecValue, 1
			;kODErrNotExportedLink, kODErrNotExportedLink,  1
			;kODErrBrokenLink, kODErrBrokenLink,  22
			;kODErrCannotRevealLink, kODErrCannotRevealLink, 39
			;kODErrCorruptLink, kODErrCorruptLink, 40
			;kODErrLinkAlreadyExported, kODErrLinkAlreadyExported, 42
			;kODErrNotImportedLink, kODErrNotImportedLink,  1
			;kODErrInvalidLinkKey, kODErrInvalidLinkKey, 1

			;kODErrBrokenLinkSource, kODErrBrokenLinkSource, 23
			;kODErrCorruptLinkSource, kODErrCorruptLinkSource, 40
			;kODErrCannotFindLinkSourceEdition, kODErrCannotFindLinkSourceEdition, 41
			;kODErrCannotFindLinkSource, kODErrCannotFindLinkSource, 55
			;kODErrAlreadyImportedLink, kODErrAlreadyImportedLink, 42
			;kODErrDocNotSaved, kODErrDocNotSaved, 24
			;kODErrNoDragManager, kODErrNoDragManager, 25
			;kODErrNoDragSystemStorage, kODErrNoDragSystemStorage, 26
			;kODErrDragItemNotFound, kODErrDragItemNotFound, 56
			;kODErrCannotAllocateDragItem, kODErrCannotAllocateDragItem, 26

			;kODErrUnknownDragImageType, kODErrUnknownDragImageType, 1
			;kODErrDragTrackingException, kODErrDragTrackingException, 27
			;kODErrNullFacetInput, kODErrNullFacetInput, 1
			;kODErrCannotCreateContainer, kODErrCannotCreateContainer, 1
			;kODErrCannotOpenContainer, kODErrCannotOpenContainer, 1
			;kODErrFocusAlreadyRegistered, kODErrFocusAlreadyRegistered,  1
			;kODErrFocusNotRegistered, kODErrFocusNotRegistered,  1
			;kODErrIllegalNullDispatchModuleInput, kODErrIllegalNullDispatchModuleInput,  1
			;kODErrCannotMarkAction, kODErrCannotMarkAction, 1
			;kODErrEmptyStack, kODErrEmptyStack, 1

			;kODErrNoBeginAction, kODErrNoBeginAction, 1
			;kODErrCannotAddAction, kODErrCannotAddAction, 1
			;kODErrCannotCreateWindow, kODErrCannotCreateWindow, 30
			;kODErrInvalidDestinationDraft,kODErrInvalidDestinationDraft, 1
			;kODErrInvalidDraftKey, kODErrInvalidDraftKey,1
			;kODErrCloningInProgress, kODErrCloningInProgress, 1
			;kODErrIllegalNullTokenInput, kODErrIllegalNullTokenInput, 1
			
			;kODErrIllegalNullShapeInput, kODErrIllegalNullShapeInput, 1
			;kODErrIllegalNullStorageUnitInput,kODErrIllegalNullStorageUnitInput, 1
			;kODErrDoesNotUndo, kODErrDoesNotLink, 4
			;kODErrPartNotWrapper, kODErrPartNotWrapper, 1
			;kODErrInvalidPersistentObject,  kODErrInvalidPersistentObject, 1

			;kODErrUndefined, kODErrUndefined, 1

			;kODErrAlreadyNotified, kODErrAlreadyNotified, 1

			;kODErrIllegalNullFacetInput, kODErrIllegalNullFacetInput, 1
			;kODErrIllegalNullFrameInput, kODErrIllegalNullFrameInput, 1
			;kODErrIllegalNullContainerInput, kODErrIllegalNullContainerInput, 1
			;kODErrIllegalNullDocumentInput, kODErrIllegalNullDocumentInput, 1
			;kODErrIllegalNullDraftInput, kODErrIllegalNullDraftInput, 1
			;kODErrIllegalNullValueTypeInput, kODErrIllegalNullValueTypeInput, 1
			;kODErrIllegalNullIDInput, kODErrIllegalNullIDInput, 1

			;kODErrValueOutOfRange, kODErrValueOutOfRange, 1

			;kODErrInsufficientInfoInParams, kODErrInsufficientInfoInParams, 1

			;kODErrInvalidPersistentFormat, kODErrInvalidPersistentFormat, 49

			;kODErrInvalidExtension, kODErrInvalidExtension, 1
			;kODErrUnknownExtension, kODErrUnknownExtension, 1

			;kODErrRefCountGreaterThanZero, kODErrRefCountGreaterThanZero, 1

			;kODErrIteratorNotInitialized, kODErrIteratorNotInitialized, 1

			;kODErrNoPromises, kODErrNoPromises, 1
			;kODErrDoesNotDrop, kODErrDoesNotDrop, 1

			;kODErrInvalidNSName, kODErrInvalidNSName, 1
			;kODErrInvalidNSType, kODErrInvalidNSType, 1

			;kODErrInvalidITextFormat, kODErrInvalidITextFormat, 1

			;kODErrInvalidPlatformShape, kODErrInvalidPlatformShape, 1

			;kODErrFacetNotFound, kODErrFacetNotFound, 1
			;kODErrCanvasHasNoOwner, kODErrCanvasHasNoOwner, 1
			;kODErrNotRootFrame, kODErrNotRootFrame, 1
			;kODErrIllegalRecursiveEmbedding, kODErrIllegalRecursiveEmbedding, 50
			;kODErrFrameHasFacets, kODErrFrameHasFacets, 1

			;kODErrInvalidLinkStatus, kODErrInvalidLinkStatus, 1

			;kODErrNotAnODToken, kODErrNotAnODToken, 1
			;kODErrCantCountFromLists, kODErrCantCountFromLists, 51

			;kODErrContainerDoesNotExist, kODErrContainerDoesNotExist, 1
			;kODErrDocumentDoesNotExist, kODErrDocumentDoesNotExist, 1
			;kODErrDraftDoesNotExist, kODErrDraftDoesNotExist, 1
			;kODErrDraftHasBeenDeleted, kODErrDraftHasBeenDeleted, 1
			;kODErrIllegalOperationOnSU, kODErrIllegalOperationOnSU, 1
			;kODErrSUValueDoesNotExist, kODErrSUValueDoesNotExist, 1
			;kODErrIllegalNonTopmostDraft, kODErrIllegalNonTopmostDraft, 1

			;kODErrValueIndexOutOfRange, kODErrValueIndexOutOfRange, 1
			;kODErrInvalidValueType, kODErrInvalidValueType, 1
			;kODErrIllegalPropertyName, kODErrIllegalPropertyName, 1
			;kODErrPropertyDoesNotExist, kODErrPropertyDoesNotExist, 1
			;kODErrMoveIntoSelf, kODErrMoveIntoSelf, 50

			;kODErrNullDestinationFrame, kODErrNullDestinationFrame, 1
			;kODErrNoLinkContent, kODErrNoLinkContent, 52
			;kODErrCannotRegisterDependent, kODErrCannotRegisterDependent, 1
			;kODErrUnknownUpdateID, kODErrUnknownUpdateID, 1
			;kODErrCannotEstablishLink, kODErrCannotEstablishLink, 52
			;kODErrNoEditionManager, kODErrNoEditionManager, 53
			;kODErrNullLinkInfoInput, kODErrNullLinkInfoInput, 1
			;kODErrNullLinkInfoResultInput, kODErrNullLinkInfoResultInput, 1

			;kODErrNullPasteAsResultInput, kODErrNullPasteAsResultInput, 1
			;kODErrBackgroundClipboardClear, kODErrBackgroundClipboardClear, 1
			;kODErrIllegalClipboardCloneKind, kODErrIllegalClipboardCloneKind, 1

			;kODErrInvalidCommandID, kODErrInvalidCommandID, 1
			;kODErrNoSysTranslationFacility, kODErrNoSysTranslationFacility, 54

			;kODErrFatalContainerError, kODErrFatalContainerError, 13
			;kODErrCannotDeferClone, kODErrCannotDeferClone, 1

/* There should be a Errors.r! */

#define dirFulErr -33
#define dskFulErr -34
#define nsvErr -35
#define ioErr -36
#define bdNamErr -37
#define fnfErr -43
#define wPrErr -44
#define fLckdErr -45
#define vLckdErr -46
#define fBsyErr -47
#define dupFNErr -48
#define opWrErr -49
#define permErr -54
#define wrPermErr -61
#define dirNFErr -120
#define memFullErr -108
#define resNotFound -192
#define resFNotFound -193
#define fragLibNotFound -2804
#define fragHadUnresolveds -2807

			;dirFulErr, dirFulErr, 80
			;dskFulErr, dskFulErr, 81
			;nsvErr, nsvErr,  82
			;ioErr, ioErr,  83
			;bdNamErr, bdNamErr,  84
			;nsvErr, nsvErr,  85
			;fnfErr, fnfErr,  86
			;wPrErr, wPrErr,  87
			;fLckdErr, fLckdErr,  88
			;vLckdErr, vLckdErr,  89
			;fBsyErr, fBsyErr,  90
			;dupFNErr, dupFNErr,  91
			;opWrErr, opWrErr,  92
			;permErr, permErr,  93
			;wrPermErr, wrPermErr,  94
			;dirNFErr, dirNFErr,  95
			;memFullErr, memFullErr,  3
			;resNotFound, resNotFound,  1
			;resFNotFound, resFNotFound,  86
			;fragLibNotFound, fragLibNotFound,  100
			;fragHadUnresolveds, fragHadUnresolveds,  100


#if ODDebug
			;minUtilsErr, maxUtilsErr, 5
			;minShellErr, maxShellErr, 7
			;minODBentoErr, maxODBentoErr, 9
			;minBentoErr, maxBentoErr, 11
#else
			;minUtilsErr, maxUtilsErr, 1
			;minShellErr, maxShellErr, 1
			;minODBentoErr, maxODBentoErr, 11
			;minBentoErr, maxBentoErr, 11
#endif
			;minUIErr, maxUIErr, 6

			;kODMinError, kODMinUsedError, 79

			;minErr, maxErr, 2
			
	}
};

/* error user explanations */
/*
   DO NOT DELETE ANY STRINGS FROM THE STR# resources. Simply empty them out.
   Like this:   ;""
   This we they don't get renumbered, we don't have to try and fix up the
   'errs' resources to point to the renumbered indexes into the STR# resources.
*/

resource 'STR#' (kODErrUserIDStrings,purgeable) {
	{		"An unknown error has occurred in OpenDoc or a part editor."
/* 2 */		;"An unknown error has occurred, probably a part editor error. Try using a different editor."
/* 3 */		;"Out of memory. Try closing some windows to make more memory available."
/* 4 */		;"The feature or action you requested is not implemented in this part editor. Try using a different editor."
/* 5 */		;"A private Utilities error has found its way to the shell. This shouldn't happen. Please file a bug."
#if ODDebug
/* 6 */		;"A private UI subsystem error has found its way to the shell. This shouldn't happen. Please file a bug."
#else
/* 6 */		;"An internal error has occurred in OpenDoc's event processing."
#endif
/* 7 */		;"A private Shell subsystem error has found its way to the shell. This shouldn't happen. Please file a bug."
/* 8 */		;"The part editor used to create the document doesnt allow embedding. Try using a different editor."
/* 9 */		;"A private Bento container suite error has found its way to the shell. This shouldn't happen. Please file a bug."
/* 10 */	;"You can't change the document because you don't have enough access privileges. See the owner of the document to have your privileges changed or make a copy of the document to make changes to it."

/* 11*/		;"An internal error has occurred in OpenDocs document storage system."
/* 12 */	;"A private Bento error has found its way to the shell. This shouldn't happen. Please file a bug."
/* 13 */	;"A fatal storage error occurred (probably out of memory or disk space.) The document file is intact, but must close immediately without saving changes. Sorry."
/* 14 */	;"A permissions error occurred. Perhaps an editor tried to modify a read-only draft or document."
#if ODDebug
/* 15 */	;"Unused slot in messages table. You shouldn't be seeing this message. Please file a bug."
#else
/* 15 */	;""
#endif
/* 16 */	;"The part editor could not be found. Try reinstalling it."
#if ODDebug
/* 17 */	;"Unused slot in messages table. You shouldn't be seeing this message. Please file a bug."
#else
/* 17 */	;""
#endif
/* 18 */	;"You cant delete drafts of this document."
#if ODDebug
/* 19 */	;"Unused slot in messages table. You shouldn't be seeing this message. Please file a bug."
/* 20 */	;"Unused slot in messages table. You shouldn't be seeing this message. Please file a bug."
#else
/* 19 */	;""
/* 20 */	;""
#endif

/* 21 */	;"Can't create the link because the source document has been closed. Try recreating the link."
/* 22 */	;"The link has unexpectedly broken. Try re-creating the link."
/* 23 */	;"The connection between the link source and all destinations has unexpectedly been broken. You'll need to re-create all links to the source."
/* 24 */	;"The link cannot be created until the link source has been saved. Save the document that contains the link source, then try again."
/* 25 */	;"Drag and Drop is not installed on your computer. Try copying and pasting your information."
/* 26 */	;"There is not enough memory to drag and drop all these items. Try dragging them one at a time."
/* 27 */	;"An unknown error occurred while dragging."
#if ODDebug
/* 28 */	;"Unused slot in messages table. You shouldn't be seeing this message. Please file a bug."
/* 29 */	;"Unused slot in messages table. You shouldn't be seeing this message. Please file a bug."
#else
/* 28 */	;""
/* 29 */	;""
#endif
/* 30 */	;"OpenDoc is out of memory and cant open any more windows. Try closing some windows to make more memory available."

#if ODDebug
/* 31 */	;"Unused slot in messages table. You shouldn't be seeing this message. Please file a bug."
/* 32 */	;"Unused slot in messages table. You shouldn't be seeing this message. Please file a bug."
#else
/* 31 */	;""
/* 32 */	;""
#endif
/* 33 */	;"The document can't be opened because you don't have the part editor that was used to create the document. Contact the vendor for the part editor."
/* 34 */	;"Cannot create a new document because every available disk is full."
/* 35 */	;"Cannot create a new document because the startup disk is full."
/* 36 */	;"Cannot save a copy of a document onto itself. Give the copy a different name."
/* 37 */	;"Cannot save a copy onto an open document."
#if ODDebug
/* 38 */	;"Unused slot in messages table. You shouldn't be seeing this message. Please file a bug."
#else
/* 38 */	;""
#endif
/* 39 */	;"Cannot find the link source because it has been deleted. Try recreating the link."
/* 40 */	;"There is a problem with this link. Try breaking the link, then creating it again."

/* 41 */	;"There is a problem with the Edition file, an intermediary file which handles links. Try breaking and re-creating the link."
/* 42 */	;"An OpenDoc linking error occurred. Break and re-create the link. Then close and re-open the document. "
#if ODDebug
/* 43 */	;"Unused slot in messages table. You shouldn't be seeing this message. Please file a bug."
/* 44 */	;"Unused slot in messages table. You shouldn't be seeing this message. Please file a bug."
#else
/* 43 */	;""
/* 44 */	;""
#endif
/* 45 */	;"The part is in use. Try again later."
/* 46 */	;"An internal memory error has occurred. Try closing the document, then opening it again."
#if ODDebug
/* 47 */	;"Unused slot in messages table. You shouldn't be seeing this message. Please file a bug."
/* 48 */	;"Unused slot in messages table. You shouldn't be seeing this message. Please file a bug."
#else
/* 47 */	;""
/* 48 */	;""
#endif
/* 49 */	;"Some data in the document cannot be read because its format is unknown."
/* 50 */	;"Cannot insert a part within itself."

/* 51 */	;"Cannot process an incoming event (possibly a script)."
/* 52 */	;"There was a problem creating this link. Please recreate the link."
/* 53 */	;"A necessary system service, the Edition Manager, is not available in your system software. To get the Edition Manager, upgrade to System Software version 7.0 or later."
/* 54 */	;"A necessary system service is not available (translation services)."
/* 55 */	;"Cannot find the link source. Trying breaking and re-creating the link."
/* 56 */	;"Cannot find the item to be dragged."
#if ODDebug
/* 57 */	;"Unused slot in messages table. You shouldn't be seeing this message. Please file a bug."
/* 58 */	;"Unused slot in messages table. You shouldn't be seeing this message. Please file a bug."
/* 59 */	;"Unused slot in messages table. You shouldn't be seeing this message. Please file a bug."
/* 60 */	;"Unused slot in messages table. You shouldn't be seeing this message. Please file a bug."

/* 61 */	;"Unused slot in messages table. You shouldn't be seeing this message. Please file a bug."
/* 62 */	;"Unused slot in messages table. You shouldn't be seeing this message. Please file a bug."
/* 63 */	;"Unused slot in messages table. You shouldn't be seeing this message. Please file a bug."
/* 64 */	;"Unused slot in messages table. You shouldn't be seeing this message. Please file a bug."
/* 65 */	;"Unused slot in messages table. You shouldn't be seeing this message. Please file a bug."
/* 66 */	;"Unused slot in messages table. You shouldn't be seeing this message. Please file a bug."
/* 67 */	;"Unused slot in messages table. You shouldn't be seeing this message. Please file a bug."
#else
/* 57 */	;""
/* 58 */	;""
/* 59 */	;""
/* 60 */	;""

/* 61 */	;""
/* 62 */	;""
/* 63 */	;""
/* 64 */	;""
/* 65 */	;""
/* 66 */	;""
/* 67 */	;""
#endif
#if ODDebug
/* 68 */	;"Unused slot in messages table. You shouldn't be seeing this message. Please file a bug."
/* 69 */	;"Unused slot in messages table. You shouldn't be seeing this message. Please file a bug."
#else
/* 68 */	;""
/* 69 */	;""
#endif
/* 70 */	;"Cannot find the Editors folder. Re-install OpenDoc and your part editors."

/* 71 */	;"Cannot find the OpenDoc Libraries Folder. Re-install OpenDoc."
#if ODDebug
/* 72 */	;"Unused slot in messages table. You shouldn't be seeing this message. Please file a bug."
/* 73 */	;"Unused slot in messages table. You shouldn't be seeing this message. Please file a bug."
#else
/* 72 */	;""
/* 73 */	;""
#endif
/* 74 */	;"Unable to create new stationery because the part editor could not be found.  "
				"Please make sure it is installed correctly."
/* 75 */	;"Cannot save as onto an open document."
#if ODDebug
/* 76 */	;"Unused slot in messages table. You shouldn't be seeing this message. Please file a bug."
/* 77 */	;"Unused slot in messages table. You shouldn't be seeing this message. Please file a bug."
/* 78 */	;"Unused slot in messages table. You shouldn't be seeing this message. Please file a bug."
/* 79 */	;"Missing an entry for an OpenDoc error code. Please file a bug."
#else
/* 76 */	;""
/* 77 */	;""
/* 78 */	;""
/* 79 */	;""
#endif
/* 80 */	;"The directory is full. Try removing files from the folder or starting a new folder."

/* 81 */	;"The disk is full. Try removing files from the disk."
/* 82 */	;"The disk does not exist."
/* 83 */	;"There is a hardware problem with the hard disk."
/* 84 */	;"A bad file name was used. The name may contain special characters that are not allowed. Try using a different name."
/* 85 */	;"The disk does not exist."
/* 86 */	;"The file was not found."
/* 87 */	;"The diskette is locked."
/* 88 */	;"The file is locked."
/* 89 */	;"The volume is locked."
/* 90 */	;"The file is already open."

/* 91 */	;"A file by that name already exists. Use a different name."
/* 92 */	;"The file is already open."
/* 93 */	;"Cannot access the file."
/* 94 */	;"Cannot access the file for writing. The folder or disk may be locked or you may not have privileges to change the file."
/* 95 */	;"The folder was not found."
#if ODDebug
/* 96 */	;"Unused slot in messages table. You shouldn't be seeing this message. Please file a bug."
/* 97 */	;"Unused slot in messages table. You shouldn't be seeing this message. Please file a bug."
/* 98 */	;"Unused slot in messages table. You shouldn't be seeing this message. Please file a bug."
/* 99 */	;"Unused slot in messages table. You shouldn't be seeing this message. Please file a bug."
#else
/* 96 */	;""
/* 97 */	;""
/* 98 */	;""
/* 99 */	;""
#endif
/* 100 */	;"Cannot find all the necessary shared libraries. Try re-installing OpenDoc."
	}
};

/* list mapping exception codes to technical explanations */

resource 'errs' (kODErrSupportID,purgeable) {
	{	whichList, 0, kODErrSupportIDStrings
			;kODErrUndefined, kODErrUndefined, 99
			;kODErrIllegalNullInput, kODErrIllegalNullInput, 2
			;kODErrObjectNotInitialized, kODErrObjectNotInitialized, 3
			;kODErrOutOfMemory, kODErrOutOfMemory, 4
			;kODErrNotImplemented, kODErrNotImplemented, 5
			;kODErrSubClassResponsibility, kODErrSubClassResponsibility, 8
			;kODErrUnsupportedExtension, kODErrUnsupportedExtension, 9
			;kODErrZeroRefCount, kODErrZeroRefCount, 10
			
			;kODErrIteratorOutOfSync, kODErrIteratorOutOfSync, 11
			;kODErrCannotEmbed, kODErrCannotEmbed, 12
			;kODErrKeyAlreadyExists, kODErrKeyAlreadyExists, 13
			;kODErrNoShapeGeometry, kODErrNoShapeGeometry, 15
			;kODErrUnsupportedFramePositionCode, kODErrUnsupportedFramePositionCode, 16
			;kODErrIllegalNullPartInput, kODErrIllegalNullPartInput, 17
			;kODErrIllegalNullTransformInput, kODErrIllegalNullTransformInput, 18
			;kODErrUnsupportedPosCode, kODErrUnsupportedPosCode, 20
			
			;kODErrInvalidPermissions, kODErrInvalidPermissions, 21
			;kODErrNoValueAtThatIndex, kODErrNoValueAtThatIndex, 22
			;kODErrCannotAddProperty, kODErrCannotAddProperty, 23
			;kODErrUnfocusedStorageUnit, kODErrUnfocusedStorageUnit, 25
			;kODErrIllegalNullPropertyInput, kODErrIllegalNullPropertyInput, 26
			;kODErrIllegalNullSUCursorInput, kODErrIllegalNullSUCursorInput, 27
			;kODErrInvalidStorageUnitRef, kODErrInvalidStorageUnitRef, 28
			;kODErrInvalidStorageUnitKey, kODErrInvalidStorageUnitKey, 30
			
			;kODErrStorageUnitNotLocked, kODErrStorageUnitNotLocked, 31
			;kODErrNoDraftProperties, kODErrNoDraftProperties, 32
			;kODErrCannotCreateFrame, kODErrCannotCreateFrame, 33
			;kODErrCannotAcquireFrame, kODErrCannotAcquireFrame, 34
			;kODErrCannotCreatePart, kODErrCannotCreatePart, 35
			;kODErrCannotAcquirePart, kODErrCannotAcquirePart, 36
			;kODErrCannotCreateLink, kODErrCannotCreateLink, 37
			;kODErrCannotAcquireLink, kODErrCannotAcquireLink, 38
			;kODErrInvalidCloneKind, kODErrInvalidCloneKind, 39
			;kODErrInvalidBelowDraft, kODErrInvalidBelowDraft, 40
			
			;kODErrCannotCollapseDrafts, kODErrCannotCollapseDrafts, 42
			;kODErrNonEmptyDraft, kODErrNonEmptyDraft, 43
			;kODErrNoPreviousDraft, kODErrNoPreviousDraft, 44
			;kODErrOutstandingDraft, kODErrOutstandingDraft, 45
			;kODErrInvalidDraftID, kODErrInvalidDraftID, 46
			;kODErrCannotChangePermissions, kODErrCannotChangePermissions, 47
			;kODErrContainerExists, kODErrContainerExists, 48
			;kODErrCannotGetExternalLink, kODErrCannotGetExternalLink, 50
			
			;kODErrNoLinkSpecValue, kODErrNoLinkSpecValue, 51
			;kODErrUnknownLinkSpecVersion, kODErrUnknownLinkSpecVersion, 52
			;kODErrCorruptLinkSpecValue, kODErrCorruptLinkSpecValue, 53
			;kODErrNotExportedLink, kODErrNotExportedLink, 54
			;kODErrBrokenLink, kODErrBrokenLink, 55
			;kODErrNotImportedLink, kODErrNotImportedLink, 56
			;kODErrBrokenLinkSource, kODErrBrokenLinkSource, 57
			;kODErrDocNotSaved, kODErrDocNotSaved, 58
			;kODErrNoDragManager, kODErrNoDragManager, 59
			;kODErrNoDragSystemStorage, kODErrNoDragSystemStorage, 60
		
			;kODErrDragItemNotFound, kODErrDragItemNotFound, 61
			;kODErrCannotAllocateDragItem, kODErrCannotAllocateDragItem, 62
			;kODErrUnknownDragImageType, kODErrUnknownDragImageType, 63
			;kODErrDragTrackingException, kODErrDragTrackingException, 64
			;kODErrFocusAlreadyRegistered, kODErrFocusAlreadyRegistered, 65
			;kODErrFocusNotRegistered, kODErrFocusNotRegistered, 66
			;kODErrIllegalNullDispatchModuleInput, kODErrIllegalNullDispatchModuleInput, 67
			;kODErrCannotMarkAction, kODErrCannotMarkAction, 68
			;kODErrEmptyStack, kODErrEmptyStack, 69
			;kODErrNoBeginAction, kODErrNoBeginAction, 70
	
			;kODErrCannotAddAction, kODErrCannotAddAction, 71
			;kODErrCannotCreateWindow, kODErrCannotCreateWindow, 72
			;kODErrInvalidObjectType, kODErrInvalidObjectType,74
			;kODErrInvalidPersistentObjectID, kODErrInvalidPersistentObjectID, 75
			;kODErrRefCountNotEqualOne, kODErrRefCountNotEqualOne, 76
			;kODErrCouldNotBindForeignDocument, kODErrCouldNotBindForeignDocument, 77
			;kODErrPartInUse, kODErrPartInUse, 78
			;kODErrAllWritableVolumesAreFull, kODErrAllWritableVolumesAreFull, 79
			;kODErrSystemVolumeIsFull, kODErrSystemVolumeIsFull, 80

			;kODErrDirectoryNotHaveWriteAccess, kODErrDirectoryNotHaveWriteAccess, 81
			;kODErrCannotSaveACopyOntoCurrentDoc, kODErrCannotSaveACopyOntoCurrentDoc, 82
			;kODErrCannotSaveACopyOntoBusyDoc, kODErrCannotSaveACopyOntoBusyDoc, 83
			;kODErrIllegalNullStorageSystemInput, kODErrIllegalNullStorageSystemInput, 84
			;kODErrCannotRevealLink, kODErrCannotRevealLink, 85
			;kODErrCorruptLink, kODErrCorruptLink, 86
			;kODErrLinkAlreadyExported, kODErrLinkAlreadyExported, 87
			;kODErrCorruptLinkSource, kODErrCorruptLinkSource, 88
			;kODErrCannotFindLinkSourceEdition, kODErrCannotFindLinkSourceEdition, 89
			;kODErrCannotFindLinkSource, kODErrCannotFindLinkSource, 90

			;kODErrAlreadyImportedLink, kODErrAlreadyImportedLink, 91
			;kODErrNullFacetInput, kODErrNullFacetInput, 92
			;kODErrInvalidFrame, kODErrInvalidFrame, 97
			;kODErrInvalidFacet, kODErrInvalidFacet, 98

			;kODErrInvalidGraphicsSystem, kODErrInvalidGraphicsSystem, 101
			;kODErrTransformErr, kODErrTransformErr, 102
			;kODErrCanvasNotFound, kODErrCanvasNotFound, 103
			;kODErrInvalidBlock, kODErrInvalidBlock, 104
			;kODErrInvalidNSName, kODErrInvalidNSName, 105
			;kODErrInvalidStorageUnit, kODErrInvalidStorageUnit, 109

			;kODErrInvalidID, kODErrInvalidID, 112
			;kODErrInconsistentCloneKind, kODErrInconsistentCloneKind, 113
			;kODErrInvalidLinkKey, kODErrInvalidLinkKey, 114
			;kODErrCannotCreateContainer, kODErrCannotCreateContainer, 115
			;kODErrCannotOpenContainer, kODErrCannotOpenContainer, 116
			;kODErrInvalidDestinationDraft,kODErrInvalidDestinationDraft, 117
			;kODErrInvalidDraftKey, kODErrInvalidDraftKey, 118
			;kODErrCloningInProgress, kODErrCloningInProgress, 119

			;kODErrIllegalNullTokenInput, kODErrIllegalNullTokenInput, 121		
			;kODErrIllegalNullShapeInput, kODErrIllegalNullShapeInput, 122
			;kODErrIllegalNullStorageUnitInput,kODErrIllegalNullStorageUnitInput, 123
			;kODErrCannotFindSystemEditorsFolder, kODErrCannotFindSystemEditorsFolder, 124
			;kODErrCannotFindOpenDocLibrariesFolder, kODErrCannotFindOpenDocLibrariesFolder, 125
			;kODErrDoesNotUndo, kODErrDoesNotLink, 126
			;kODErrPartNotWrapper, kODErrPartNotWrapper, 127
			;kODErrInvalidPersistentObject,  kODErrInvalidPersistentObject, 128
			;kODErrEditorLibraryNotFound, kODErrEditorLibraryNotFound, 130
			;kODErrCannotSaveAsOntoBusyDoc, kODErrCannotSaveAsOntoBusyDoc, 131

			;kODErrAlreadyNotified, kODErrAlreadyNotified, 132

			;kODErrIllegalNullFacetInput, kODErrIllegalNullFacetInput, 133
			;kODErrIllegalNullFrameInput, kODErrIllegalNullFrameInput, 134
			;kODErrIllegalNullContainerInput, kODErrIllegalNullContainerInput, 135
			;kODErrIllegalNullDocumentInput, kODErrIllegalNullDocumentInput, 136
			;kODErrIllegalNullDraftInput, kODErrIllegalNullDraftInput, 137
			;kODErrIllegalNullValueTypeInput, kODErrIllegalNullValueTypeInput, 138
			;kODErrIllegalNullIDInput, kODErrIllegalNullIDInput, 139

			;kODErrValueOutOfRange, kODErrValueOutOfRange, 140

			;kODErrInsufficientInfoInParams, kODErrInsufficientInfoInParams, 141

			;kODErrInvalidPersistentFormat, kODErrInvalidPersistentFormat, 142

			;kODErrInvalidExtension, kODErrInvalidExtension, 143
			;kODErrUnknownExtension, kODErrUnknownExtension, 144

			;kODErrRefCountGreaterThanZero, kODErrRefCountGreaterThanZero, 145

			;kODErrIteratorNotInitialized, kODErrIteratorNotInitialized, 146

			;kODErrNoPromises, kODErrNoPromises, 147
			;kODErrDoesNotDrop, kODErrDoesNotDrop, 148

			;kODErrInvalidNSName, kODErrInvalidNSName, 149
			;kODErrInvalidNSType, kODErrInvalidNSType, 150

			;kODErrInvalidITextFormat, kODErrInvalidITextFormat, 151

			;kODErrInvalidPlatformShape, kODErrInvalidPlatformShape, 152

			;kODErrFacetNotFound, kODErrFacetNotFound, 153
			;kODErrCanvasHasNoOwner, kODErrCanvasHasNoOwner, 154
			;kODErrNotRootFrame, kODErrNotRootFrame, 155
			;kODErrIllegalRecursiveEmbedding, kODErrIllegalRecursiveEmbedding, 156
			;kODErrFrameHasFacets, kODErrFrameHasFacets, 157

			;kODErrInvalidLinkStatus, kODErrInvalidLinkStatus, 158

			;kODErrNotAnODToken, kODErrNotAnODToken, 159
			;kODErrCantCountFromLists, kODErrCantCountFromLists, 160

			;kODErrContainerDoesNotExist, kODErrContainerDoesNotExist, 161
			;kODErrDocumentDoesNotExist, kODErrDocumentDoesNotExist, 162
			;kODErrDraftDoesNotExist, kODErrDraftDoesNotExist, 163
			;kODErrDraftHasBeenDeleted, kODErrDraftHasBeenDeleted, 164
			;kODErrIllegalOperationOnSU, kODErrIllegalOperationOnSU, 165
			;kODErrSUValueDoesNotExist, kODErrSUValueDoesNotExist, 166
			;kODErrIllegalNonTopmostDraft, kODErrIllegalNonTopmostDraft, 167

			;kODErrValueIndexOutOfRange, kODErrValueIndexOutOfRange, 168
			;kODErrInvalidValueType, kODErrInvalidValueType, 169
			;kODErrIllegalPropertyName, kODErrIllegalPropertyName, 170
			;kODErrPropertyDoesNotExist, kODErrPropertyDoesNotExist, 171
			;kODErrMoveIntoSelf, kODErrMoveIntoSelf, 172

			;kODErrNullDestinationFrame, kODErrNullDestinationFrame, 173
			;kODErrNoLinkContent, kODErrNoLinkContent, 174
			;kODErrCannotRegisterDependent, kODErrCannotRegisterDependent, 175
			;kODErrUnknownUpdateID, kODErrUnknownUpdateID, 176
			;kODErrCannotEstablishLink, kODErrCannotEstablishLink, 177
			;kODErrNoEditionManager, kODErrNoEditionManager, 178
			;kODErrNullLinkInfoInput, kODErrNullLinkInfoInput, 179
			;kODErrNullLinkInfoResultInput, kODErrNullLinkInfoResultInput, 180

			;kODErrNullPasteAsResultInput, kODErrNullPasteAsResultInput, 181
			;kODErrBackgroundClipboardClear, kODErrBackgroundClipboardClear, 182
			;kODErrIllegalClipboardCloneKind, kODErrIllegalClipboardCloneKind, 183

			;kODErrInvalidCommandID, kODErrInvalidCommandID, 184
			;kODErrNoSysTranslationFacility, kODErrNoSysTranslationFacility, 206

			;kODErrFatalContainerError, kODErrFatalContainerError, 96
			;kODErrCannotDeferClone, kODErrCannotDeferClone, 100

			;dirFulErr, dirFulErr, 185
			;dskFulErr, dskFulErr, 186
			;nsvErr, nsvErr,  187
			;ioErr, ioErr,  188
			;bdNamErr, bdNamErr,  189
			;nsvErr, nsvErr,  190
			;fnfErr, fnfErr,  191
			;wPrErr, wPrErr,  192
			;fLckdErr, fLckdErr,  193
			;vLckdErr, vLckdErr,  194
			;fBsyErr, fBsyErr,  195
			;dupFNErr, dupFNErr,  196
			;opWrErr, opWrErr,  197
			;permErr, permErr,  198
			;wrPermErr, wrPermErr,  199
			;dirNFErr, dirNFErr,  200
			;memFullErr, memFullErr,  201
			;resNotFound, resNotFound,  202
			;resFNotFound, resFNotFound,  203
			;fragLibNotFound, fragLibNotFound,  204
			;fragHadUnresolveds, fragHadUnresolveds,  205


			;minUtilsErr, maxUtilsErr, 24
			;minShellErr, maxShellErr, 29
			;minODBentoErr, maxODBentoErr, 41
			;minBentoErr, maxBentoErr, 49
			;minUIErr, maxUIErr, 95

			;kODMinError, kODMinUsedError, 1

			;minErr, maxErr, 210
	}
};

/* error technical explanations */
/*
   DO NOT DELETE ANY STRINGS FROM THE STR# resources. Simply empty them out.
   Like this:   ;""
   This we they don't get renumbered, we don't have to try and fix up the
   'errs' resources to point to the renumbered indexes into the STR# resources.
*/

resource 'STR#' (kODErrSupportIDStrings,purgeable) {
	{	 "Unknown OpenDoc error. No name available. Please file a bug."
		,"kODErrIllegalNullInput"
		,"kODErrObjectNotInitialized"
		,"kODErrOutOfMemory"
		,"kODErrNotImplemented"
		,"kODErrReadErr"
		,"kODErrWriteErr"
		,"kODErrSubClassResponsibility"
		,"kODErrUnsupportedExtension"
/* 10 */,"kODErrZeroRefCount"
		
		,"kODErrIteratorOutOfSync"
		,"kODErrCannotEmbed"
		,"kODErrKeyAlreadyExists"
		,"kODErrWrongNameSpaceType"
		,"kODErrNoShapeGeometry"
		,"kODErrUnsupportedFramePositionCode"
		,"kODErrIllegalNullPartInput"
		,"kODErrIllegalNullTransformInput"
		,"kODErrPartHasNoSemanticInterface"
/* 20 */,"kODErrUnsupportedPosCode"
		
		,"kODErrInvalidPermissions"
		,"kODErrNoValueAtThatIndex"
		,"kODErrCannotAddProperty"
		,"Private Utilities error."
		,"kODErrUnfocusedStorageUnit"
		,"kODErrIllegalNullPropertyInput"
		,"kODErrIllegalNullSUCursorInput"
		,"kODErrInvalidStorageUnitRef"
		,"Private Shell error."
/* 30 */,"kODErrInvalidStorageUnitKey"
		
		,"kODErrStorageUnitNotLocked"
		,"kODErrNoDraftProperties"
		,"kODErrCannotCreateFrame"
		,"kODErrCannotAcquireFrame"
		,"kODErrCannotCreatePart"
		,"kODErrCannotAcquirePart"
		,"kODErrCannotCreateLink"
		,"kODErrCannotAcquireLink"
		,"kODErrInvalidCloneKind"
/* 40 */,"kODErrInvalidBelowDraft"
		
		,"Private Bento container suite error."
		,"kODErrCannotCollapseDrafts"
		,"kODErrNonEmptyDraft"
		,"kODErrNoPreviousDraft"
		,"kODErrOutstandingDraft"
		,"kODErrInvalidDraftID"
		,"kODErrCannotChangePermissions"
		,"kODErrContainerExists"
		,"Private Bento error."
/* 50 */,"kODErrCannotGetExternalLink"
		
		,"kODErrNoLinkSpecValue"
		,"kODErrUnknownLinkSpecVersion"
		,"kODErrCorruptLinkSpecValue"
		,"kODErrNotExportedLink"
		,"kODErrBrokenLink"
		,"kODErrNotImportedLink"
		,"kODErrBrokenLinkSource"
		,"kODErrDocNotSaved"
		,"kODErrNoDragManager"
/* 60 */,"kODErrNoDragSystemStorage"
		
		,"kODErrDragItemNotFound"
		,"kODErrCannotAllocateDragItem"
		,"kODErrUnknownDragImageType"
		,"kODErrDragTrackingException"
		,"kODErrFocusAlreadyRegistered"
		,"kODErrFocusNotRegistered"
		,"kODErrIllegalNullDispatchModuleInput"
		,"kODErrCannotMarkAction"
		,"kODErrEmptyStack"
/* 70 */,"kODErrNoBeginAction"
		
		,"kODErrCannotAddAction"
		,"kODErrCannotCreateWindow"	
		,"kODErrSOMException"
		,"kODErrInvalidObjectType"
		,"kODErrInvalidPersistentObjectID"
		,"kODErrRefCountNotEqualOne"
		,"kODErrCoundNotBindForeignDocument"
		,"kODErrPartInUse"
		,"kODErrAllWritableVolumesAreFull"
/* 80 */,"kODErrSystemVolumeIsFull"

		,"kODErrDirectoryNotHaveWriteAccess"
		,"kODErrCannotSaveACopyOntoCurrentDoc"
		,"kODErrCannotSaveACopyOntoBusyDoc"
		,"kODErrIllegalNullStorageSystemInput"
		,"kODErrCannotRevealLink"
		,"kODErrCorruptLink"
		,"kODErrLinkAlreadyExported"
		,"kODErrCorruptLinkSource"
		,"kODErrCannotFindLinkSourceEdition"
/* 90 */,"kODErrCannotFindLinkSource"

		,"kODErrAlreadyImportedLink"
		,"kODErrNullFacetInput"
		,"kODErrFileClosed"
		,"kODErrFileOpened"
		,"Private UI error."
		,"kODErrFatalContainerError"
		,"kODErrInvalidFrame"
		,"kODErrInvalidFacet"
		,"kODErrUndefined"
/*100 */,"kODErrCannotDeferClone"

		,"kODErrInvalidGraphicsSystem"
		,"kODErrTransformErr"
		,"Unused message slot. Please report this."
		,"kODErrMemoryProblem"
		,"kODErrInvalidNSName"
		,"Unused message slot. Please report this."
		,"Unused message slot. Please report this."
		,"Unused message slot. Please report this."
		,"kODErrInvalidStorageUnit"
/*110 */,"kODErrInvalidType"

		,"Unused message slot. Please report this."
		,"kODErrInvalidID"
		,"kODErrInconsistentCloneKind"
		,"kODErrInvalidLinkKey"
		,"kODErrCannotCreateContainer"
		,"kODErrCannotOpenContainer"	
		,"kODErrInvalidDestinationDraft"	
		,"kODErrInvalidDraftKey"	
		,"kODErrCloningInProgress"	
/*120 */,"kODErrNoDocumentProperties"	

		,"kODErrIllegalNullTokenInput"	
		,"kODErrIllegalNullShapeInput"	
		,"kODErrIllegalNullStorageUnitInput"	
		,"kODErrCannotFindSystemEditorsFolder"	
		,"kODErrCannotFindOpenDocLibrariesFolder"	
		,"kODErrDoesNotUndo"	
		,"kODErrPartNotWrapper"	
		,"kODErrInvalidPersistentObject"	
		,"kODErrCantLoadSOMClass"
/*130 */,"kODErrEditorLibraryNotFound"

 		,"kODErrCannotSaveAsOntoBusyDoc"
 		,"kODErrAlreadyNotified"
 		,"kODErrIllegalNullFacetInput"
 		,"kODErrIllegalNullFrameInput"
 		,"kODErrIllegalNullContainerInput"
 		,"kODErrIllegalNullDocumentInput"
 		,"kODErrIllegalNullDraftInput"
 		,"kODErrIllegalNullValueTypeInput"
 		,"kODErrIllegalNullIDInput"
/*140 */,"kODErrValueOutOfRange"

		,"kODErrInsufficientInfoInParams"
 		,"kODErrInvalidPersistentFormat"
 		,"kODErrInvalidExtension"
 		,"kODErrUnknownExtension"
 		,"kODErrRefCountGreaterThanZero"
 		,"kODErrIteratorNotInitialized"
 		,"kODErrNoPromises"
 		,"kODErrDoesNotDrop"
 		,"kODErrInvalidNSName"
/*150 */,"kODErrInvalidNSType"

 		,"kODErrInvalidITextFormat"
		,"kODErrInvalidPlatformShape"
 		,"kODErrFacetNotFound"
 		,"kODErrCanvasHasNoOwner"
 		,"kODErrNotRootFrame"
 		,"kODErrIllegalRecursiveEmbedding"
 		,"kODErrFrameHasFacets"
 		,"kODErrInvalidLinkStatus"
 		,"kODErrNotAnODToken"
/*160 */ ,"kODErrCantCountFromLists"

 		,"kODErrContainerDoesNotExist"
 		,"kODErrDocumentDoesNotExist"
		,"kODErrDraftDoesNotExist"
 		,"kODErrDraftHasBeenDeleted"
 		,"kODErrIllegalOperationOnSU"
 		,"kODErrSUValueDoesNotExist"
 		,"kODErrIllegalNonTopmostDraft"
 		,"kODErrValueIndexOutOfRange"
 		,"kODErrInvalidValueType"
/*170 */,"kODErrIllegalPropertyName"

 		,"kODErrPropertyDoesNotExist"
 		,"kODErrMoveIntoSelf"
 		,"kODErrNullDestinationFrame"
		,"kODErrNoLinkContent"
 		,"kODErrCannotRegisterDependent"
 		,"kODErrUnknownUpdateID"
 		,"kODErrCannotEstablishLink"
 		,"kODErrNoEditionManager"
 		,"kODErrNullLinkInfoInput"
/*180 */,"kODErrNullLinkInfoResultInput"

		,"kODErrNullPasteAsResultInput"
 		,"kODErrBackgroundClipboardClear"
 		,"kODErrIllegalClipboardCloneKind"
 		,"kODErrInvalidCommandID"
		,"dirFulErr"
		,"dskFulErr"
		,"nsvErr"
		,"ioErr"
		,"bdNamErr"
/*190 */,"nsvErr"

		,"fnfErr"
		,"wPrErr"
		,"fLckdErr"
		,"vLckdErr"
		,"fBsyErr"
		,"dupFNErr"
		,"opWrErr"
		,"permErr"
		,"wrPermErr"
/*200 */,"dirNFErr"

		,"memFullErr"
		,"resNotFound"
		,"resFNotFound"
		,"fragLibNotFound"
		,"fragHadUnresolveds"
		,"kODErrNoSysTranslationFacility"
		,"Unused message slot. Please report this."
		,"Unused message slot. Please report this."
		,"Unused message slot. Please report this."
/*210 */,"Unknown error, outside of OpenDoc error range. No name available."
	}
};


