/*
	File:		LinkDlgs.h

	Contains:	function declarations for Paste As and Link Info dialogs

	Owned by:	Craig Carper

	Copyright:	 1993 - 1995 by Apple Computer, Inc., all rights reserved.

	Change History (most recent first):

		<11>	 8/29/95	DM		1221778, 1276165: [ShowLinkUpdateAlert]
									drop count arg in ShowLinkUpdateAlert
		<10>	 5/26/95	VL		1251403: Multithreading naming support.
		 <9>	 3/22/95	CC		1230322: Changed parameter to
									ShowPasteAsDialog().
		 <8>	  3/9/95	CC		1221471: Added changesAllowed parameter to
									ShowLinkDestinationInfo() and
									ShowLinkSourceInfo().
		 <7>	  3/3/95	CC		1223861 Added ShowLinkStatusAlert().
		 <6>	 2/15/95	CC		1220327 Added InitLinkDialogs().
		 <5>	  2/7/95	CC		 1211295: ShowPasteAsDialog() takes an
									isMove parameter.
		 <4>	 9/20/94	CC		1160232 - Added ShowLinkUpdateAlert().
		 <3>	 8/10/94	CC		1179943 - LinkInfo dialog functions take
									ODBaseLink and ODBaseLinkSource parameters.
		 <2>	  8/2/94	CC		#1178169 - ShowPasteAsDialog() takes
									ODFacet* instead of ODFrame*.
		 <1>	 6/24/94	CC		first checked in

		------------------- CC ---- Converted to SOM

		 <8>	 5/27/94	jpa		Remove ASLM dependency [1165267]
		 <7>	  5/9/94	MB		#1162181: Changes necessary to install MMM.
		 <6>	  4/6/94	CC		XMPTypeInfo: Changed field "creationDate"
									to "creationTime" and "modificationDate" to
									"changeTime". (1154722)
		 <5>	  4/5/94	CC		Removed unused declaration of class
									XMPTypeSet. (1153046)
		 <4>	 3/25/94	JA		Added missing #include (1147588)
		 <3>	 3/24/94	CC		ShowLinkSourceInfo() takes XMPChangeID
									parameter; XMPPasteAsResult specifies
									original kind when selected kind is a
									translation (1153068).
		 <2>	 3/17/94	CC		Added ShowLinkSourceInfo(),
									ShowLinkDestinationInfo(), XMPLinkAction,
									XMPLinkResult, and XMPLinkInfo. (1151329)
		 <1>	 3/14/94	CC		first checked in
		 <0>	 3/14/94	CC		Declaration of ShowPasteAsDialog (1144253).

	To Do:
*/

#ifndef _LINKDLGS_
#define _LINKDLGS_

#ifndef _ODTYPES_
#include <ODTypes.h>
#endif

#ifndef __TIME_H__
#include <time.h>
#endif

//==============================================================================
// Classes used by this interface
//==============================================================================

class ODStorageUnit;
class ODFacet;
class ODBaseLinkSource;
class ODBaseLink;
class ODSession;

//==============================================================================
// Data Interchange functions
//==============================================================================

//-------------------------------------------------------------------------------------
// InitLinkDialogs
//-------------------------------------------------------------------------------------

void InitLinkDialogs(ODSession* session);

//-------------------------------------------------------------------------------------
// ShowPasteAsDialog
//-------------------------------------------------------------------------------------

ODBoolean ShowPasteAsDialog(
				ODBoolean				canPasteLink,
				ODPasteAsMergeSetting	mergeSetting,
				ODBoolean				isMove,
				ODStorageUnit*			contentSU,
				ODFacet*				facet,
				ODTypeToken				viewType,
				ODPasteAsResult*		result);

//-------------------------------------------------------------------------------------
// ShowLinkSourceInfo
//-------------------------------------------------------------------------------------

ODBoolean ShowLinkSourceInfo(
				ODBaseLinkSource*	linkSource,
				ODUpdateID			change,
				ODBoolean			changesAllowed,
				ODLinkInfoResult*	infoResult);
				
//-------------------------------------------------------------------------------------
// ShowLinkDestinationInfo
//-------------------------------------------------------------------------------------

ODBoolean ShowLinkDestinationInfo(
				ODBaseLink*			link,
				ODLinkInfo* 		info,
				ODBoolean			changesAllowed,
				ODLinkInfoResult*	infoResult);

//------------------------------------------------------------------------------
// ShowLinkUpdateAlert
//------------------------------------------------------------------------------

ODBoolean ShowLinkUpdateAlert( );

//------------------------------------------------------------------------------
// ShowLinkStatusAlert
//------------------------------------------------------------------------------

void ShowLinkStatusAlert(ODError error);

#endif // _LINKDLGS_

