/*
	File:		LinkDlgs.r

	Contains:	resources for data interchange dialogs

	Owned by:	Craig Carper

	Copyright:	 1994 - 1996 by Apple Computer, Inc., all rights reserved.

	Change History (most recent first):

		 <2>	  6/7/96	eeh		T10017: move most stuff to LnkDlgEx.r
		<19>	10/16/95	CC		GM 1293229: 'MENU' resources must be
									nonpurgeable.
									GM 1292157: Changed "No Kinds" to
									"Unknown"; "None Available" to "None for
									this Kind".
		<18>	 9/20/95	T		1285201 GM: Use
									alertPositionParentWindowScreen for dialogs
		<17>	 8/23/95	CC		1278840: Added kPasteAsEmbedEditorText
									item.
		<16>	 7/28/95	CC		1242615: Made all resources purgeable.
									1242576: Removed resources for Others
									dialog.
		<15>	 4/27/95	CC		1243348: Removed BreakLinkAlert resources.
		<14>	 4/25/95	CC		1242555: Entered as bug 1242615 to check
									for purgeable resources.
									1242555: $5 entered as bug 1242617.
		<13>	 3/27/95	CC		1229640: Added items for editor popup and
									static text.
		<12>	 3/17/95	CC		1220339: Changed title of Translation
									Choices dialog.
		<11>	  3/9/95	CC		1220342: Tweeked layout of others dialog.
		<10>	  3/3/95	CC		1223861 Added link status and link edition
									alert resources.
		 <9>	 2/24/95	CC		1153783: Changed link edition filename
									template string.
		 <8>	 2/15/95	CC		1194705 Changed titles of link info dialogs
									to "Link Destination Info" and "Link Source
									Info".
									1196831 Added strings for constructing size
									of links.
									1153942 Changed "Modified:" to "Updated:"
									in Link Info dialogs
									1220324 Removed static text items for
									labeling radio buttons.
									1191537 Revised Link Update alert
									resources.
									1220318 Data Interchange Dialogs need to
									use 'finf' resource for localization
									1191537 Need better wording in circular
									link alert.
									1220324 Added 'CNTL' resources for controls
									that use window font.
		 <7>	  2/7/95	CC		1211295: Added two items to PasteAs dialog:
									kPasteAsKindLabel kPasteAsKindText.
		 <6>	  2/1/95	CC		1153347: Added 'STR ' resource for
									notification.
		 <5>	 1/10/95	CC		1153930: Added translation choices to Paste
									As dialog.
		 <4>	 10/7/94	CC		1160232 - Changes to link update alerts.
		 <3>	 9/20/94	CC		1160232 - Added resources for link cycle
									Alert dialog.
		 <2>	 7/31/94	CC		Initialize kind string of link destination
									dialog to "(Unknown)".
		 <1>	 6/24/94	CC		first checked in
		 <3>	  6/3/94	CC		Fixed dctb resources.
		 <2>	 3/17/94	CC		Added resources for ShowLinkSourceInfo()
									and ShowLinkDestinationInfo(). (1151329)
		 <1>	 3/14/94	CC		first checked in
		 <0>	 3/14/94	CC		Resources for Paste As dialog (1144253).

*/
#define _USE_DIALOGS_LIB_

// Picture items are used for the merge and embed icons because 32x32 ICONs
// don't work well.  The desired icon size is 19(high)x16(width).  If the
// display item is the desired size, the ICON is scaled.  If the display item
// is 32x32, the ICONs are drawn AFTER their radio buttons and can obscure them,
// even though the radio buttons are later in the item sequence.

#define SystemSevenOrBetter 1			// we want the extended types
#define	SystemSevenOrLater	1			// Types.r uses this variable

#define oldTemp		// Needed to use old-style 'dctb' resource
#include "Types.r"
#undef oldTemp		// End of 'dctb' fix

#include "SysTypes.r"


#include "LinkDefs.h"

#ifndef _USE_DIALOGS_LIB_

#define kDialogHeight	223
#define kDialogWidth	222

#define kTopBegin					  7
#define kLeading					  4
#define kLeftBegin					 10

#define kLargeToSmallVerticalOffset	  1
#define kSmallLineHeight			 12
#define kRightStop					213
#define kBottomStop					214
#define kStdButtonHeight			 20
#define kOkCancelWidth				 60
#define kOkCancelSep				 13
#define kCheckboxHeight				 14
#define kCheckboxWidth				 14
#define kRadioButtonHeight			 14
#define kRadioButtonWidth			 14
#define kCheckboxVertSep			  4
#define kCheckboxHorizSep			  4
#define kRadioBtnVertSep			  3
#define kRadioBtnHorizSep			  4
#define kPopupHeight				 19

#define kPasteWithLinkTextTop		kTopBegin+kLargeToSmallVerticalOffset
#define kPasteWithLinkTextLeft		kLeftBegin+kCheckboxWidth+kCheckboxHorizSep
#define kPasteWithLinkTextWidth		78
#define kPasteWithLinkTextRight		kPasteWithLinkTextLeft+kPasteWithLinkTextWidth

#define kUpdateLinksTop				kTopBegin+kCheckboxHeight+kLeading
#define kUpdateLinksLeft			kLeftBegin+kCheckboxWidth+kCheckboxHorizSep
#define kUpdateLinksWidth			59

#define kAutomaticRadioBtnTop		kTopBegin+kCheckboxHeight+kRadioBtnVertSep
#define kAutomaticRadioBtnLeft		kLeftBegin+kCheckboxWidth+kCheckboxHorizSep+kUpdateLinksWidth+kCheckboxHorizSep
#define kAutomaticTextTop			kAutomaticRadioBtnTop+kLargeToSmallVerticalOffset
#define kAutomaticTextLeft			kAutomaticRadioBtnLeft+kRadioButtonWidth+kRadioBtnHorizSep
#define kAutomaticTextWidth			68
#define kAutomaticTextRight			kAutomaticTextLeft+kAutomaticTextWidth
#define kAutoManRadioBtnSep			1

#define kManualRadioBtnTop			kAutomaticRadioBtnTop+kRadioButtonHeight+kAutoManRadioBtnSep
#define kManualRadioBtnLeft			kAutomaticRadioBtnLeft
#define kManualTextTop				kManualRadioBtnTop+kLargeToSmallVerticalOffset
#define kManualTextLeft				kManualRadioBtnLeft+kRadioButtonWidth+kRadioBtnHorizSep
#define kManualTextWidth			46

#define kManualTextRight			kManualTextLeft+kManualTextWidth

/* #define kHorizSepTop				kManualTextTop+kSmallLineHeight+kLeading */
#define kHorizSepTop				62

#define kAtDestTextTop				72

#define kMergePictTop				90
#define kMergePictHeight			19
#define kMergePictWidth				16

#define kMergeRadioBtnTop			kMergePictTop+2		
#define kMergeRadioBtnLeft			kLeftBegin+kMergePictWidth+kCheckboxHorizSep
#define kMergeTextTop				kMergeRadioBtnTop+kLargeToSmallVerticalOffset
#define kMergeTextLeft				kMergeRadioBtnLeft+kRadioButtonWidth+kRadioBtnHorizSep
#define kMergeTextWidth				100
#define kMergeTextRight				kMergeTextLeft+kMergeTextWidth

#define kEmbedPictTop				kMergePictTop+kMergePictHeight+3
#define kEmbedPictHeight			kMergePictHeight
#define kEmbedPictWidth				kMergePictWidth

#define kEmbedRadioBtnTop			kEmbedPictTop+2
#define kEmbedRadioBtnLeft			kMergeRadioBtnLeft
#define kEmbedTextTop				kEmbedRadioBtnTop+kLargeToSmallVerticalOffset
#define kEmbedTextLeft				kMergeTextLeft
#define kEmbedTextWidth				51
#define kEmbedTextRight				kEmbedTextLeft+kEmbedTextWidth

#define kEmbedPopupTop				kEmbedRadioBtnTop-1
#define kEmbedPopupLeft				kEmbedTextLeft+kEmbedTextWidth

#define kKindPopupTop				140
#define kKindPopupLeft				kMergeRadioBtnLeft-2
#define kKindTextWidth				33

#define kEditorPopupTop				kKindPopupTop+21
#define kEditorPopupLeft			kMergeRadioBtnLeft-10
#define kEditorTextWidth			41


//-------------------------------------------------------------------------------------
// Paste As Dialog
//-------------------------------------------------------------------------------------

resource 'DLOG' (kPasteAsDlgID, "Paste As", purgeable) {
	{0, 0, kDialogHeight, kDialogWidth},
	movableDBoxProc,
	invisible,
	noGoAway,
	0x0,
	kPasteAsDlgID,
	"Paste As",
	alertPositionParentWindowScreen
};

/* Color table for Paste As Dialog */
resource 'dctb' (kPasteAsDlgID, "Paste As", purgeable) {
	0,
	0,
	{
		wContentColor,0xFFFF,0xFFFF,0xFFFF
	}
};

/* Item list for Paste As dialog */
resource 'DITL' (kPasteAsDlgID,  "Paste As", purgeable) {
	{	/* array DITLarray: 23 elements */
		/* [1] */
		{ kBottomStop-kStdButtonHeight, 
		  kRightStop-kOkCancelWidth, 
		  kBottomStop, 
		  kRightStop
		},
		Button {
			enabled,
			"Paste"
		},
		/* [2] */
		{ kBottomStop-kStdButtonHeight,
		  kRightStop-kOkCancelWidth-kOkCancelSep-kOkCancelWidth,
		  kBottomStop,
		  kRightStop-kOkCancelWidth-kOkCancelSep
		},
		Button {
			enabled,
			"Cancel"
		},
		
		/* [3] Paste with Link checkbox */
		{ kTopBegin,
		  kLeftBegin,
		  kTopBegin+kCheckboxHeight,
		  kPasteWithLinkTextRight
		},
		Control {
			enabled,
			kPasteAsCheckboxCtrlID
		},

		/* [4] Update Links label (active appearance) */
		{ kUpdateLinksTop,
		  kUpdateLinksLeft,
		  kUpdateLinksTop+kLargeToSmallVerticalOffset+kSmallLineHeight,
		  kUpdateLinksLeft+kUpdateLinksWidth
		},
		StaticText {
			disabled,
			"Get Updates:"
		},

		/* [5] Update Links label (inactive appearance) */
		{ kUpdateLinksTop,
		  kUpdateLinksLeft,
		  kUpdateLinksTop+kLargeToSmallVerticalOffset+kSmallLineHeight,
		  kUpdateLinksLeft+kUpdateLinksWidth
		},
		StaticText {
			disabled,
			"Get Updates:"
		},

		/* [6] Automatic update radio button */
		{ kAutomaticRadioBtnTop,
		  kAutomaticRadioBtnLeft,
		  kAutomaticRadioBtnTop + kRadioButtonHeight,
		  kAutomaticTextRight
		},
		Control {
			enabled,
			kPasteAsAutoRadioBtnCtrlID
		},

		/* [7] Manual update radio button */
		{ kManualRadioBtnTop,
		  kManualRadioBtnLeft,
		  kManualRadioBtnTop + kRadioButtonHeight,
		  kManualTextRight
		},
		Control {
			enabled,
			kPasteAsManualRadioBtnCtrlID
		},

		/* [8] Horizontal Separator */
		{ kHorizSepTop,
		  kLeftBegin,
		  kHorizSepTop + 1,
		  kRightStop
		},
		UserItem {
			enabled
		},

		/* [9] At the destination text */
		{ kAtDestTextTop,
		  kLeftBegin,
		  kAtDestTextTop + kSmallLineHeight,
		  kRightStop
		},
		StaticText {
			disabled,
			"At the Destination:"
		},

		/* [10] Merge Pict */
		{ kMergePictTop,
		  kLeftBegin,
		  kMergePictTop + kMergePictHeight,
		  kLeftBegin + kMergePictWidth
		},
		Picture {
			enabled,
			kPasteAsMergePictID
		},

		/* [11] User item to disable previous item */
		{ 0,0,0,0 },
		UserItem {
			disabled,
		},

		/* [12] Merge radio button */
		{ kMergeRadioBtnTop,
		  kMergeRadioBtnLeft,
		  kMergeRadioBtnTop + kRadioButtonHeight,
		  kMergeTextRight
		},
		Control {
			enabled,
			kPasteAsMergeRadioBtnCtrlID
		},

		/* [13] Embed Pict */
		{ kEmbedPictTop,
		  kLeftBegin,
		  kEmbedPictTop + kEmbedPictHeight,
		  kLeftBegin + kEmbedPictWidth
		},
		Picture {
			enabled,
			kPasteAsEmbedPictID
		},

		/* [14] User item to disable previous item */
		{ 0,0,0,0 },
		UserItem {
			disabled,
		},

		/* [15] Embed radio button */
		/* The label of the Embed As radio button overlaps the title of the Embed As popup. */
		/* The radio button item must be before the popup item, so mouse clicks over the */
		/* label are handled by the radio button item, not the popup item. */
		/* Both controls draw the text; this causes problems when the popup is disabled, */
		/* since it will dim the text and the radio button will appear inactive. */
		{ kEmbedRadioBtnTop,
		  kEmbedRadioBtnLeft,
		  kEmbedRadioBtnTop + kRadioButtonHeight,
		  kEmbedTextRight
		},
		Control {
			enabled,
			kPasteAsEmbedRadioBtnCtrlID
		},

		/* [16] Embed As popup menu */
		{ kEmbedPopupTop,
		  kEmbedPopupLeft,
		  kEmbedPopupTop + kPopupHeight,
		  kRightStop
		},
		Control {
			enabled,
			kPasteAsEmbedPopupCntl
		},

		/* [17] Kind popup menu */
		{ kKindPopupTop,
		  kKindPopupLeft,
		  kKindPopupTop + kPopupHeight,
		  kRightStop
		},
		Control {
			enabled,
			kPasteAsKindPopupCntl
		},

		/* [18] Kind label */
		/* Used when the popup is unavailable because selecting a kind is not allowed */
		{ kKindPopupTop + 2,
		  kKindPopupLeft + 3,
		  kKindPopupTop + kSmallLineHeight,
		  kKindPopupLeft + 35
		},
		StaticText {
			disabled,
			"Kind:"
		},

		/* [19] Current kind */
		/* Used when the popup is unavailable because selecting a kind is not allowed */
		{ kKindPopupTop + 2,
		  kKindPopupLeft + /* arbitrary */ 47,
		  kKindPopupTop + kPopupHeight,
		  kRightStop
		},
		StaticText {
			disabled,
			""
		},

		/* [20] Editor popup menu */
		{ kEditorPopupTop,
		  kEditorPopupLeft,
		  kEditorPopupTop + kPopupHeight,
		  kRightStop
		},
		Control {
			enabled,
			kPasteAsEditorPopupCntl
		},

		/* [21] Editor label */
		/* Used when the popup is unavailable because selecting a Editor is not allowed */
		{ kEditorPopupTop + 2,
		  kEditorPopupLeft + 3,
		  kEditorPopupTop + kSmallLineHeight,
		  kEditorPopupLeft + 38
		},
		StaticText {
			disabled,
			"Editor:"
		},

		/* [22] Current Editor */
		/* Used when the popup is unavailable because selecting a Editor is not allowed */
		{ kEditorPopupTop + 2,
		  kEditorPopupLeft + 55,
		  kEditorPopupTop + kPopupHeight,
		  kRightStop
		},
		StaticText {
			disabled,
			"Unknown"
		},

		/* [23] Embedded Editor */
		/* Used when the popup is unavailable because a frame is being moved */
		{ kEditorPopupTop + 2,
		  kEditorPopupLeft + 55,
		  kEditorPopupTop + kPopupHeight,
		  kRightStop
		},
		StaticText {
			disabled,
			"Unknown"
		},

		/* [24] Outline around default button */
		{ kBottomStop-kStdButtonHeight-1-4, 
		  kRightStop-kOkCancelWidth-3-4, 
		  kBottomStop-1+4, 
		  kRightStop-3+4
		},
		UserItem {
			disabled
		}
	}
};

/* Color table for Paste As Dialog items */
data 'ictb' (kPasteAsDlgID, "Paste As") {
	$"0000 0000" /* [1]  OK button is unaffected							*/
	$"0000 0000" /* [2]  Cancel button is unaffected						*/
	$"0000 0000" /* [3]  Paste with Link checkbox is unaffected				*/
	$"0000 0000" /* [4]  Update Links label (enabled) is unaffected			*/
	$"4000 0060" /* [5]  Update Links label (disabled) change mode			*/
	$"0000 0000" /* [6]  Automatic update radio button is unaffected		*/
	$"0000 0000" /* [7]  Manual update radio button is unaffected			*/
	$"0000 0000" /* [8]  Horizontal Separator is unaffected					*/
	$"0000 0000" /* [9]  At the destination text is unaffected				*/
	$"0000 0000" /* [10] Merge Pict is unaffected							*/
	$"0000 0000" /* [11] User item for disabling is unaffected				*/
	$"0000 0000" /* [12] Merge radio button is unaffected					*/
	$"0000 0000" /* [13] Embed Pict is unaffected							*/
	$"0000 0000" /* [14] User item for disabling is unaffected				*/
	$"0000 0000" /* [15] Embed radio button is unaffected					*/
	$"0000 0000" /* [16] Embed As popup menu is unaffected					*/
	$"0000 0000" /* [17] Kind popup menu is unaffected						*/
	$"0000 0000" /* [18] 													*/
	$"0000 0000" /* [19] 													*/
	$"0000 0000" /* [20] 													*/
	$"0000 0000" /* [21] 													*/
	$"0000 0000" /* [22] 													*/
	$"0000 0000" /* [23] 													*/
	$"0000 0000" /* [24] 													*/
	/* Text style record */
    $"0003 0100 000A"      /* Geneva, Bold, 10 */
    $"0000 0000 0000"      /* fore color       */
    $"0000 0000 0000"      /* back color       */
	$"0031"                /* mode = grayishTextOr = 49 */
};

/* Window font for Paste As Dialog items */
resource 'finf' (kPasteAsDlgID, purgeable) {
	{
		3,			/* Font Number, 3 = Geneva	*/
		plain,		/* 		Font Style		*/
		10			/* 		Font Size		*/
	}
};

resource 'CNTL' (kPasteAsCheckboxCtrlID, purgeable) {
		{ kTopBegin,
		  kLeftBegin,
		  kTopBegin+kCheckboxHeight,
		  kPasteWithLinkTextRight
		},
		0,
		visible,
		1,
		0,
		checkboxProcUseWFont,
		0,							/* Refcon */
		"Paste with Link"
};

resource 'CNTL' (kPasteAsAutoRadioBtnCtrlID, purgeable) {
		{ kAutomaticRadioBtnTop,
		  kAutomaticRadioBtnLeft,
		  kAutomaticRadioBtnTop + kRadioButtonHeight,
		  kAutomaticTextRight
		},
		0,
		visible,
		1,
		0,
		radioButProcUseWFont,
		0,							/* Refcon */
		"Automatically"
};

resource 'CNTL' (kPasteAsManualRadioBtnCtrlID, purgeable) {
		{ kManualRadioBtnTop,
		  kManualRadioBtnLeft,
		  kManualRadioBtnTop + kRadioButtonHeight,
		  kManualTextRight
		},
		0,
		visible,
		1,
		0,
		radioButProcUseWFont,
		0,							/* Refcon */
		"Manually"
};

resource 'CNTL' (kPasteAsMergeRadioBtnCtrlID, purgeable) {
		{ kMergeRadioBtnTop,
		  kMergeRadioBtnLeft,
		  kMergeRadioBtnTop + kRadioButtonHeight,
		  kMergeTextRight
		},
		0,
		visible,
		1,
		0,
		radioButProcUseWFont,
		0,							/* Refcon */
		"Merge with Contents"
};

resource 'CNTL' (kPasteAsEmbedRadioBtnCtrlID, purgeable) {
		{ kEmbedRadioBtnTop,
		  kEmbedRadioBtnLeft,
		  kEmbedRadioBtnTop + kRadioButtonHeight,
		  kEmbedTextRight
		},
		0,
		visible,
		1,
		0,
		radioButProcUseWFont,
		0,							/* Refcon */
		"Embed as:"
};

resource 'PICT' (kPasteAsMergePictID, "Merge Icon", purgeable) {
	97,
	{0, 0, 19, 16},
	$"1101 A000 8201 000A 0000 0000 0020 0020"
	$"9000 0200 0000 0000 1300 1000 0000 0000"
	$"1300 1000 0000 0000 1300 1000 00FF FF80"
	$"0180 0192 4980 0180 0193 F980 0180 019F"
	$"F980 0180 019F C980 0180 0192 4980 0180"
	$"01FF FFA0 0083 FF"
};

resource 'PICT' (kPasteAsEmbedPictID, "Embed Icon", purgeable) {
	97,
	{0, 0, 19, 16},
	$"1101 A000 8201 000A 0000 0000 0020 0020"
	$"9000 0200 0000 0000 1300 1000 0000 0000"
	$"1300 1000 0000 0000 1300 1000 00FF FF80"
	$"0180 0192 4980 0180 0193 F982 0982 0992"
	$"0982 0982 0993 F980 0180 0192 4980 0180"
	$"01FF FFA0 0083 FF"
};

// The "Embed as" popup has no title, because the radio button label occupies the
// same space.  If the radio button isn't selected, the popup is disabled.  If the
// popup had a title which exactly overlayed the radio button label, the title
// would be dimmed and the radio button would appear disabled.
resource 'CNTL' (kPasteAsEmbedPopupCntl, "Embed as", purgeable) {
		{ kEmbedPopupTop,
		  kEmbedPopupLeft,
		  kEmbedPopupTop + kPopupHeight,
		  kRightStop
		},
		popupTitleLeftJust,
		visible,
		0,							/* zero width title; radio button provides title */
		kPasteAsEmbedPopupMenu,		/* resource id of MENU resource */
		popupMenuCDEFproc+popupUseWFont+popupFixedWidth,
		0,							/* Refcon */
		""
};

resource 'CNTL' (kPasteAsKindPopupCntl, "Kind", purgeable) {
		{ kKindPopupTop,
		  kKindPopupLeft,
		  kKindPopupTop + kPopupHeight,
		  kRightStop
		},
		popupTitleLeftJust,
		visible,
		kKindTextWidth,				/* width of title in pixels     */
		kPasteAsKindPopupMenu,		/* resource id of MENU resource */
		popupMenuCDEFproc+popupUseWFont+popupFixedWidth,
		0,							/* Refcon */
		"Kind:"
};

resource 'CNTL' (kPasteAsEditorPopupCntl,"Editor", purgeable) {
		{ kEditorPopupTop,
		  kEditorPopupLeft,
		  kEditorPopupTop + kPopupHeight,
		  kRightStop
		},
		popupTitleLeftJust,
		visible,
		kEditorTextWidth,				/* width of title in pixels     */
		kPasteAsEditorPopupMenu,		/* resource id of MENU resource */
		popupMenuCDEFproc+popupUseWFont+popupFixedWidth,
		0,							/* Refcon */
		"Editor:"
};

resource 'MENU' (kPasteAsEmbedPopupMenu, "Embed as", nonpurgeable) {	// 'MENU' resources must not be purgeable [cc]
	kPasteAsEmbedPopupMenu,
	textMenuProc,
	allEnabled,
	enabled,
	"",
	{	/* array: 4 elements */
		"Frame", noIcon, noKey, noMark, plain,
		"Large Icon", noIcon, noKey, noMark, plain,
		"Small Icon", noIcon, noKey, noMark, plain,
		"Thumbnail", noIcon, noKey, noMark, plain
	}
};

resource 'MENU' (kPasteAsKindPopupMenu, "Kind", nonpurgeable) {
	kPasteAsKindPopupMenu,
	textMenuProc,
	allEnabled,
	enabled,
	"",
	{	/* array: 1 elements */
		"Translate to", noIcon, noKey, noMark, italic
	}
};

resource 'MENU' (kPasteAsEditorPopupMenu, "Editor", nonpurgeable) {
	kPasteAsEditorPopupMenu,
	textMenuProc,
	allEnabled,
	enabled,
	"",
	{	/* array: 0 elements */
		"<Your editor goes here>", noIcon, noKey, noMark, plain
	}
};

//-------------------------------------------------------------------------------------
// Link Source Info Dialog
//-------------------------------------------------------------------------------------

#define kLinkDlgLabelRight 87
#define kLinkDlgInfoLeft   97
#define kLinkDlgInfoRight  256

resource 'DLOG' (kLinkSrcInfoDlgID, "Link Source Info", purgeable) {
	{74, 114, 235, 487},
	movableDBoxProc,
	visible,
	goAway,
	0x0,
	kLinkSrcInfoDlgID,
	"Link Source Info",
	alertPositionParentWindowScreen
};

resource 'DITL' (kLinkSrcInfoDlgID, "Link Source Info", purgeable) {
	{	/* array DITLarray: 16 elements */
		/* [1] kLinkSrcOKBtn */
		{131, 268, 151, 359},
		Button {
			enabled,
			"OK"
		},
		/* [2] kLinkSrcCancelBtn */
		{100, 264, 120, 363},
		Button {
			enabled,
			"Cancel"
		},
		/* [3] Outline around default button */
		{ 131-1-4, 268-3-4, 151-1+4, 359-3+4},
		UserItem {
			disabled
		},
		/* [4] kLinkSrcSeparator */
		{76, 264, 77, 363},
		UserItem {
			disabled
		},
		/* [5] NOT USED IN THIS DIALOG; used as a placeholder to keep item numbers
		 *                              the same between LinkSrc and LinkDst
		 */
		{0, 0, 0, 0},
		Button {
			enabled,
			"Find Source"
		},
		/* [6] kLinkSrcUpdateBtn */
		{34, 264, 54, 363},
		Button {
			enabled,
			"Update Now"
		},
		/* [7] kLinkSrcBreakLinkBtn */
		{7, 264, 27, 363},
		Button {
			enabled,
			"Break Link"
		},
		/* [8] kLinkSrcManualRadioBtn */
		{87, kLinkDlgInfoLeft-3, 102, 172},
		Control {
			enabled,
			kManualRadioBtnCtrlID
		},
		/* [9] kLinkSrcOnSaveRadioBtn */
		{71, kLinkDlgInfoLeft-3, 86, 172},
		Control {
			enabled,
			kOnSaveRadioBtnCtrlID
		},
		/* [10] */
		{71, 8, 87, kLinkDlgLabelRight+1},
		StaticText {
			disabled,
			"Send Updates:"
		},
		/* [11] Modified Date */
		{50, kLinkDlgInfoLeft, 66, kLinkDlgInfoRight},
		StaticText {
			disabled,
			"^2  ^3"
		},
		/* [12] Modified Label */
		{50, 38, 66, kLinkDlgLabelRight},
		StaticText {
			disabled,
			"Updated:"
		},
		/* [13] Creation Date */
		{29, kLinkDlgInfoLeft, 45, kLinkDlgInfoRight},
		StaticText {
			disabled,
			"^0  ^1"
		},
		/* [14] Created Label */
		{29, 39, 45, kLinkDlgLabelRight},
		StaticText {
			disabled,
			"Created:"
		},
		/* [15] Kind */
		{8, kLinkDlgInfoLeft, 24, kLinkDlgInfoRight},
		StaticText {
			disabled,
			"Unknown"
		},
		/* [16] Kind Label */
		{8, 56, 24, kLinkDlgLabelRight},
		StaticText {
			disabled,
			"Kind:"
		}
	}
};

resource 'dctb' (kLinkSrcInfoDlgID, purgeable) {
	0,
	0,
	{
		wContentColor,0xFFFF,0xFFFF,0xFFFF
	}
};

data 'ictb' (kLinkSrcInfoDlgID, purgeable) {
	$"0000 0000" /* [1]  */
	$"0000 0000" /* [2]  */
	$"0000 0000" /* [3]  */
	$"0000 0000" /* [4]  */
	$"0000 0000" /* [5]  */
	$"0000 0000" /* [6]  */
	$"0000 0000" /* [7]  */
	$"0000 0000" /* [8]  */
	$"0000 0000" /* [9]  */
	$"0002 0040" /* [10] Static text label is Geneva 10 bold */
	$"0000 0000" /* [11] */
	$"0002 0040" /* [12] Static text label is Geneva 10 bold */
	$"0000 0000" /* [13] */
	$"0002 0040" /* [14] Static text label is Geneva 10 bold */
	$"0000 0000" /* [15] */
	$"0002 0040" /* [16] Static text label is Geneva 10 bold */
	/* Text style record */
    $"0001 0100 000A"      /* application font, Bold (high byte), size 10 */
    $"0000 0000 0000"      /* fore color        */
    $"0000 0000 0000 0000" /* back color + mode */
};

resource 'finf' (kLinkSrcInfoDlgID, purgeable) {
	{
		3,			/* Font Number, 3 = Geneva	*/
		plain,		/* 		Font Style		*/
		10			/* 		Font Size		*/
	}
};
//-------------------------------------------------------------------------------------
// Link Destination Info Dialog
//-------------------------------------------------------------------------------------

resource 'DLOG' (kLinkDstInfoDlgID, "Link Destination Info", purgeable) {
	{74, 114, 235, 487},
	movableDBoxProc,
	visible,
	goAway,
	0x0,
	kLinkDstInfoDlgID,
	"Link Destination Info",
	alertPositionParentWindowScreen
};

resource 'DITL' (kLinkDstInfoDlgID, "Link Destination Info", purgeable) {
	{	/* array DITLarray: 16 elements */
		/* [1] kLinkDstOKBtn */
		{131, 268, 151, 359},
		Button {
			enabled,
			"OK"
		},
		/* [2] kLinkDstCancelBtn */
		{100, 264, 120, 363},
		Button {
			enabled,
			"Cancel"
		},
		/* [3] Outline around default button */
		{ 131-1-4, 268-3-4, 151-1+4, 359-3+4},
		UserItem {
			disabled
		},
		/* [4] kLinkDstSeparator */
		{90, 264, 91, 363},
		UserItem {
			disabled
		},
		/* [5] kLinkDstFindSrcBtn */
		{61, 264, 81, 363},
		Button {
			enabled,
			"Find Source"
		},
		/* [6] kLinkDstUpdateBtn */
		{34, 264, 54, 363},
		Button {
			enabled,
			"Update Now"
		},
		/* [7] kLinkDstBreakLinkBtn */
		{7, 264, 27, 363},
		Button {
			enabled,
			"Break Link"
		},
		/* [8] kLinkDstManualRadioBtn */
		{87, kLinkDlgInfoLeft-3, 102, 172},
		Control {
			enabled,
			kManualRadioBtnCtrlID
		},
		/* [9] kLinkDstAutoRadioBtn */
		{71, kLinkDlgInfoLeft-3, 86, 192},
		Control {
			enabled,
			kAutomaticRadioBtnCtrlID
		},
		/* [10] */
		{71, 16, 87, kLinkDlgLabelRight},
		StaticText {
			disabled,
			"Get Updates:"
		},
		/* [11] Modified Date */
		{50, kLinkDlgInfoLeft, 66, kLinkDlgInfoRight},
		StaticText {
			disabled,
			"^2  ^3"
		},
		/* [12] Modified Label */
		{50, 38, 66, kLinkDlgLabelRight},
		StaticText {
			disabled,
			"Updated:"
		},
		/* [13] Creation Date */
		{29, kLinkDlgInfoLeft, 45, kLinkDlgInfoRight},
		StaticText {
			disabled,
			"^0  ^1"
		},
		/* [14] Created Label */
		{29, 39, 45, kLinkDlgLabelRight},
		StaticText {
			disabled,
			"Created:"
		},
		/* [15] Kind */
		{8, kLinkDlgInfoLeft, 24, kLinkDlgInfoRight},
		StaticText {
			disabled,
			"Unknown"
		},
		/* [16] Kind Label */
		{8, 56, 24, kLinkDlgLabelRight},
		StaticText {
			disabled,
			"Kind:"
		}
	}
};

resource 'dctb' (kLinkDstInfoDlgID, purgeable) {
	0,
	0,
	{
		wContentColor,0xFFFF,0xFFFF,0xFFFF
	}
};

data 'ictb' (kLinkDstInfoDlgID) {
	$"0000 0000" /* [1]  */
	$"0000 0000" /* [2]  */
	$"0000 0000" /* [3]  */
	$"0000 0000" /* [4]  */
	$"0000 0000" /* [5]  */
	$"0000 0000" /* [6]  */
	$"0000 0000" /* [7]  */
	$"0000 0000" /* [8]  */
	$"0000 0000" /* [9]  */
	$"0002 0040" /* [10] Static text label is Geneva 10 bold */
	$"0000 0000" /* [11] */
	$"0002 0040" /* [12] Static text label is Geneva 10 bold */
	$"0000 0000" /* [13] */
	$"0002 0040" /* [14] Static text label is Geneva 10 bold */
	$"0000 0000" /* [15] */
	$"0002 0040" /* [16] Static text label is Geneva 10 bold */
	/* Text style record */
    $"0001 0100 000A"      /* application font, Bold (high byte), size 10 */
    $"0000 0000 0000"      /* fore color        */
    $"0000 0000 0000 0000" /* back color + mode */
};

resource 'finf' (kLinkDstInfoDlgID, purgeable) {
	{
		3,			/* Font Number, 3 = Geneva	*/
		plain,		/* 		Font Style		*/
		10			/* 		Font Size		*/
	}
};

resource 'CNTL' (kAutomaticRadioBtnCtrlID, purgeable) {
		{90, kLinkDlgInfoLeft-3, 105, 192},
		0,
		visible,
		1,
		0,
		radioButProcUseWFont,
		0,							/* Refcon */
		"Automatically"
};

resource 'CNTL' (kOnSaveRadioBtnCtrlID, purgeable) {
		{90, kLinkDlgInfoLeft-3, 105, 172},
		0,
		visible,
		1,
		0,
		radioButProcUseWFont,
		0,							/* Refcon */
		"On Save"
};

resource 'CNTL' (kManualRadioBtnCtrlID, purgeable) {
		{106, kLinkDlgInfoLeft-3, 121, 172},
		0,
		visible,
		1,
		0,
		radioButProcUseWFont,
		0,							/* Refcon */
		"Manually"
};

#endif /* #ifndef _USE_DIALOGS_LIB_ */

//-------------------------------------------------------------------------------------
// Link Update Alert
//-------------------------------------------------------------------------------------

#define kLinkUpdateTextLines 5
#define kLinkUpdateTextLineHeight 16
#define kLinkUpdateTextHeight kLinkUpdateTextLines*kLinkUpdateTextLineHeight
#define kLinkUpdateLeft 78
#define kLinkUpdateTextWidth 245
#define kLinkUpdateRight kLinkUpdateLeft+kLinkUpdateTextWidth
#define kLinkUpdateBtnTop 13+kLinkUpdateTextHeight+13
#define kLinkUpdateBtnBottom kLinkUpdateBtnTop+20
#define kLinkUpdateBtnWidth 70
#define kLinkUpdateContinueBtnLeft kLinkUpdateRight-kLinkUpdateBtnWidth
#define kLinkUpdateStopBtnLeft kLinkUpdateRight-kLinkUpdateBtnWidth-13-kLinkUpdateBtnWidth

resource 'ALRT' (kLinkUpdateAlertID, "Link Update", purgeable) {
	{0, 0, kLinkUpdateBtnBottom+13, kLinkUpdateRight+13},
	kLinkUpdateAlertID,
	{	/* array: 4 elements */
		/* [1] */
		Ok, visible, sound1,
		/* [2] */
		Ok, visible, sound1,
		/* [3] */
		Ok, visible, sound1,
		/* [4] */
		Ok, visible, sound1
	},
	alertPositionParentWindowScreen
};

resource 'DITL' (kLinkUpdateAlertID, "Link Update", purgeable) {
	{	/* array DITLarray: 4 elements */
		/* [1] */
		{kLinkUpdateBtnTop, kLinkUpdateContinueBtnLeft, kLinkUpdateBtnBottom, kLinkUpdateRight},
		Button {
			enabled,
			"Update"
		},
		/* [2] */
		{kLinkUpdateBtnTop, kLinkUpdateStopBtnLeft, kLinkUpdateBtnBottom, kLinkUpdateStopBtnLeft+kLinkUpdateBtnWidth},
		Button {
			enabled,
			"Stop"
		},
		/* [3] */
		{13, 23, 45, 55},
		Icon {
			disabled,
			2
		},
		/* [4] */
		{13, kLinkUpdateLeft, 13+kLinkUpdateTextHeight, kLinkUpdateRight},
		StaticText {
			disabled,
			"Editing a link source has caused an update loop.\n"
			"\n"
			"Press Stop to interrupt updating, or press "
			"Update to continue."
		}
	}
};

//-------------------------------------------------------------------------------------
// Link Status Alert
//-------------------------------------------------------------------------------------

#define kLinkStatusTextLines 6
#define kLinkStatusTextLineHeight 16
#define kLinkStatusTextHeight kLinkStatusTextLines*kLinkStatusTextLineHeight
#define kLinkStatusLeft 78
#define kLinkStatusTextWidth 245
#define kLinkStatusRight kLinkStatusLeft+kLinkStatusTextWidth
#define kLinkStatusBtnTop 13+kLinkStatusTextHeight+13
#define kLinkStatusBtnBottom kLinkStatusBtnTop+20
#define kLinkStatusBtnWidth 70
#define kLinkStatusOKBtnLeft kLinkStatusRight-kLinkStatusBtnWidth

resource 'ALRT' (kLinkStatusAlertID, "Link Status", purgeable) {
	{0, 0, kLinkStatusBtnBottom+13, kLinkStatusRight+13},
	kLinkStatusAlertID,
	{	/* array: 4 elements */
		/* [1] */
		Ok, visible, sound1,
		/* [2] */
		Ok, visible, sound1,
		/* [3] */
		Ok, visible, sound1,
		/* [4] */
		Ok, visible, sound1
	},
	alertPositionParentWindowScreen
};

resource 'DITL' (kLinkStatusAlertID, "Link Status", purgeable) {
	{	/* array DITLarray: 3 elements */
		/* [1] */
		{kLinkStatusBtnTop, kLinkStatusOKBtnLeft, kLinkStatusBtnBottom, kLinkStatusRight},
		Button {
			enabled,
			"OK"
		},
		/* [2] */
		{13, 23, 45, 55},
		Icon {
			disabled,
			2
		},
		/* [3] */
		{13, kLinkStatusLeft, 13+kLinkStatusTextHeight, kLinkStatusRight},
		StaticText {
			disabled,
			"The link information may not be accurate, "
			"because a problem was encountered getting information about the "
			"source of the link.\n"
			"\n"
			"(Error ^0)"
		}
	}
};

//-------------------------------------------------------------------------------------
// Missing Link Edition Alert
//-------------------------------------------------------------------------------------

#define kLinkEditionTextLines 6
#define kLinkEditionTextLineHeight 16
#define kLinkEditionTextHeight kLinkEditionTextLines*kLinkEditionTextLineHeight
#define kLinkEditionLeft 78
#define kLinkEditionTextWidth 305
#define kLinkEditionRight kLinkEditionLeft+kLinkEditionTextWidth
#define kLinkEditionBtnTop 13+kLinkEditionTextHeight+13
#define kLinkEditionBtnBottom kLinkEditionBtnTop+20
#define kLinkEditionBtnWidth 70
#define kLinkEditionOKBtnLeft kLinkEditionRight-kLinkEditionBtnWidth

resource 'ALRT' (kLinkEditionAlertID, "Link Edition", purgeable) {
	{0, 0, kLinkEditionBtnBottom+13, kLinkEditionRight+13},
	kLinkEditionAlertID,
	{	/* array: 4 elements */
		/* [1] */
		Ok, visible, sound1,
		/* [2] */
		Ok, visible, sound1,
		/* [3] */
		Ok, visible, sound1,
		/* [4] */
		Ok, visible, sound1
	},
	alertPositionParentWindowScreen
};

resource 'DITL' (kLinkEditionAlertID, "Link Edition", purgeable) {
	{	/* array DITLarray: 3 elements */
		/* [1] */
		{kLinkEditionBtnTop, kLinkEditionOKBtnLeft, kLinkEditionBtnBottom, kLinkEditionRight},
		Button {
			enabled,
			"OK"
		},
		/* [2] */
		{13, 23, 45, 55},
		Icon {
			disabled,
			2
		},
		/* [3] */
		{13, kLinkEditionLeft, 13+kLinkEditionTextHeight, kLinkEditionRight},
		StaticText {
			disabled,
			"The source of this link could not be found.\n"
			"\n"
			"If you have not broken the link at the source, you may be able to "
			"get a more recent update by opening the document containing "
			"the source of this link."
		}
	}
};

//-------------------------------------------------------------------------------------
// Color pattern used by link dialogs
//-------------------------------------------------------------------------------------

data 'ppat' (kLinkDlgsppatID) {
	$"0001 0000 001C 0000 004E 0000 0000 FFFF"
	$"0000 0000 AA55 AA55 AA55 AA55 0000 0000"
	$"8001 0000 0000 0008 0008 0000 0000 0000"
	$"0000 0048 0000 0048 0000 0000 0001 0001"
	$"0001 0000 0000 0000 0056 0000 0000 55AA"
	$"55AA 55AA 55AA 0000 0000 0000 0001 0000"
	$"7777 7777 7777 0001 8888 8888 8888"
};

//-------------------------------------------------------------------------------------
// Strings used by the Link Manager
//-------------------------------------------------------------------------------------

resource 'STR ' (kODLinkMgrCannotShowSourceID, purgeable) {
	"Cannot show source of link (error ^0)."
};

resource 'STR ' (kODLinkMgrEditionTemplateID, purgeable) {
	"^0 Link ^1"
};

//-------------------------------------------------------------------------------------
// Strings used by Link Info dialogs
//-------------------------------------------------------------------------------------

resource 'STR ' (kODLinkInfoStrUnknownID, purgeable) {
	"Unknown"
};

//-------------------------------------------------------------------------------------
// Strings used by Paste As dialogs
//-------------------------------------------------------------------------------------

resource 'STR ' (kODPasteAsNoUserKindID, purgeable) {
	"Unknown kind<I;"
};
