/*
	File:		TransDlg.r

	Contains:	resources for Translation dialog

	Owned by:	Craig Carper

	Copyright:	 1993 - 1995 by Apple Computer, Inc., all rights reserved.

	Change History (most recent first):

		 <9>	10/16/95	CC		GM 1292157: Changed text in Translate to
									dialog.
		 <8>	 9/11/95	CG		#1229981 FB2: Changed to smart quotes in
									DITLs.
		 <7>	  9/7/95	CG		#1229981 FB2:NoPart Alert when no
									translators are installed.
		 <6>	 8/24/95	CC		1279966,1279963: Separate resources for
									Part translation dialog.
		 <5>	 3/28/95	CC		1229640: Added alternate title for
									translate dialog.
		 <4>	 3/27/95	CC		1229640: Added 'finf' resource, changed
									dialog text.
		 <3>	 1/10/95	CG		#1207407: "OK" button is initially in
									active.
		 <2>	12/13/94	CG		#1187128: Dialog clean up.
		 <1>	11/17/94	CG		first checked in
	To Do:
*/

#define SystemSevenOrBetter 1			// we want the extended types
#define	SystemSevenOrLater	1			// Types.r uses this variable

#include "Types.r"
#include "SysTypes.r"

#include "TrnDlgDf.h"

#define kODTransDataItemRight	316

/* Translation Dialog */
resource 'DLOG' (kODTransDlgID, "Translate") {
	{kODTransDialogTop, kODTransDialogLeft, 
		kODTransDialogBottom, kODTransDialogRight},
	dBoxProc,
	invisible,
	noGoAway,
	0x0,
	kODTransDlgID,
	"Part Translation",
	alertPositionParentWindow
};

/* Color table for Translation Dialog */
resource 'dctb' (kODTransDlgID, "Translate") {
	0,
	0,
	{
		wContentColor,0xFFFF,0xFFFF,0xFFFF
	}
};

/* Item list for Translation dialog */
resource 'DITL' (kODTransDlgID, "Translate") {
	{	/* array DITLarray: 9 elements */
		/* [1] kODTransOKBtn */
		{91, 177, 91+20, 177+59},
		Button {
			enabled,
			"OK"
		},
		/* [2] kODTransCancelBtn */
		{91, 105, 91+20, 105+59},
		Button {
			enabled,
			"Cancel"
		},
		/* [3] kODTransTitle */
		{kItemOnEdgeWhtSp, kItemOnEdgeWhtSp,
			kItemOnEdgeWhtSp+kODTransStaticTxtHeight, 
			kItemOnEdgeWhtSp+200},
		StaticText {
			disabled,
			"Translate to:"
		},
		/* [4] kODTransKindPopup */
		{34, 20, 34+16, 20+200},
		Control {
			enabled,
			kODTransKindCntl
		},
		/* [5] kODTransEditorPopup */
		{55, 20, 55+16, 20+200},
		Control {
			enabled,
			kODTransEditorCntl
		},
		/* [6] kODTransEditorLabel */
		{55, 30, 55+16, 75},
		StaticText {
			disabled,
			"Editor:"
		},
		/* [7] kODTransEditorText */
		{55, 80, 55+16, 20+200},
		StaticText {
			disabled,
			"Unknown"
		},
		/* [8] kODTransNoEditorText */
		{55, 80, 55+16, 20+200},
		StaticText {
			disabled,
			"None for this Kind"
		},
		/* [9] Outline around default button */
		{ 91-1-4, 
		  177-3-4, 
		  91+20-1+4, 
		  177+59-3+4
		},
		UserItem {
			disabled
		}
	}
};

/* Color table for Translation Dialog items */
data 'ictb' (kODTransDlgID, "Translate") {
	$"0000 0000" /* [1]  */
	$"0000 0000" /* [2]  */
	$"0002 0024" /* [3]  Bold this item */
	$"0000 0000" /* [4]  */
	$"0000 0000" /* [5]  */
	$"0000 0000" /* [6]  */
	$"0000 0000" /* [7]  */
	$"0000 0000" /* [8]  */
	$"0000 0000" /* [9]  */
	/* Text style record */
    $"0000 0100 0000"      /* Bold              */
    $"0000 0000 0000"      /* fore color        */
    $"0000 0000 0000 0000" /* back color + mode */
};

/* Window font for Translation dialog */
resource 'finf' (kODTransDlgID, "", purgeable) {
	{
		3,			/* Font Number, 3 = Geneva	*/
		plain,		/* 		Font Style		*/
		10			/* 		Font Size		*/
	}
};

/* Part Translation Dialog */
resource 'DLOG' (kODPartTransDlgID, "Translate") {
	{kODTransDialogTop, kODTransDialogLeft, 
		kODTransDialogBottom, kODTransDialogRight},
	dBoxProc,
	invisible,
	noGoAway,
	0x0,
	kODPartTransDlgID,
	"Part Translation",
	alertPositionParentWindow
};

/* Color table for Part Translation Dialog */
resource 'dctb' (kODPartTransDlgID, "Translate") {
	0,
	0,
	{
		wContentColor,0xFFFF,0xFFFF,0xFFFF
	}
};

/* Item list for Part Translation dialog */
resource 'DITL' (kODPartTransDlgID, "Translate") {
	{	/* array DITLarray: 9 elements */
		/* [1] kODTransOKBtn */
		{91, 157, 91+20, 157+79},
		Button {
			enabled,
			"Translate"
		},
		/* [2] kODTransCancelBtn */
		{91, 85, 91+20, 85+59},
		Button {
			enabled,
			"Cancel"
		},
		/* [3] kODTransTitle */
		{kItemOnEdgeWhtSp, kItemOnEdgeWhtSp,
			kItemOnEdgeWhtSp+kODTransStaticTxtHeight, 
			kItemOnEdgeWhtSp+200},
		StaticText {
			disabled,
			"Translate this part to:"
		},
		/* [4] kODTransKindPopup */
		{34, 20, 34+16, 20+200},
		Control {
			enabled,
			kODTransKindCntl
		},
		/* [5] kODTransEditorPopup */
		{55, 20, 55+16, 20+200},
		Control {
			enabled,
			kODTransEditorCntl
		},
		/* [6] kODTransEditorLabel */
		{55, 30, 55+16, 75},
		StaticText {
			disabled,
			"Editor:"
		},
		/* [7] kODTransEditorText */
		{55, 80, 55+16, 20+200},
		StaticText {
			disabled,
			"Unknown"
		},
		/* [8] kODTransNoEditorText */
		{55, 80, 55+16, 20+200},
		StaticText {
			disabled,
			"None for this Kind"
		},
		/* [9] Outline around default button */
		{ 91-1-4, 
		  177-3-4, 
		  91+20-1+4, 
		  177+59-3+4
		},
		UserItem {
			disabled
		}
	}
};

/* Color table for Part Translation Dialog items */
data 'ictb' (kODPartTransDlgID, "Translate") {
	$"0000 0000" /* [1]  */
	$"0000 0000" /* [2]  */
	$"0002 0024" /* [3]  Bold this item */
	$"0000 0000" /* [4]  */
	$"0000 0000" /* [5]  */
	$"0000 0000" /* [6]  */
	$"0000 0000" /* [7]  */
	$"0000 0000" /* [8]  */
	$"0000 0000" /* [9]  */
	/* Text style record */
    $"0000 0100 0000"      /* Bold              */
    $"0000 0000 0000"      /* fore color        */
    $"0000 0000 0000 0000" /* back color + mode */
};

/* Window font for Part Translation dialog */
resource 'finf' (kODPartTransDlgID, "", purgeable) {
	{
		3,			/* Font Number, 3 = Geneva	*/
		plain,		/* 		Font Style		*/
		10			/* 		Font Size		*/
	}
};

resource 'CNTL' (kODTransKindCntl, "Kind") {
		{34, 20, 34+16, 20+200},
		popupTitleBold+popupTitleRightJust,
		visible,
		47,					/* width of title in pixels     */
		kODTransKindMenu,	/* resource id of MENU resource */
		popupMenuCDEFproc+popupUseWFont+popupFixedWidth,
		0,					/* Refcon */
		"Kind:"
};

resource 'CNTL' (kODTransEditorCntl, "Editor") {
		{55, 20, 55+16, 20+200},
		popupTitleBold+popupTitleRightJust,
		visible,
		47,					/* width of title in pixels     */
		kODTransEditorMenu,	/* resource id of MENU resource */
		popupMenuCDEFproc+popupUseWFont+popupFixedWidth,
		0,					/* Refcon */
		"Editor:"
};

resource 'MENU' (kODTransKindMenu, "Kind") {
	kODTransKindMenu,
	textMenuProc,
	allEnabled,
	enabled,
	"Kind",
	{	/* array: 0 elements */
	}
};

resource 'MENU' (kODTransEditorMenu, "Editor") {
	kODTransEditorMenu,
	textMenuProc,
	allEnabled,
	enabled,
	"Editor",
	{	/* array: 0 elements */
	}
};


resource 'ALRT' (kODNoTransAlert, "No editor/translator") {
	{40, 40, 124, 383},
	kODNoTransAlert,
	{	/* array: 4 elements */
		/* [1] */
		OK, visible, sound1,
		/* [2] */
		OK, visible, sound1,
		/* [3] */
		OK, visible, sound1,
		/* [4] */
		OK, visible, sound1
	}
#if SystemSevenOrLater
	,
	centerParentWindowScreen
#endif
};

resource 'DITL' (kODNoTransAlert, "No editor/translator") {
	{	/* array DITLarray: 4 elements */
		/* [1] */
		{53, 270, 73, 328},
		Button {
			enabled,
			"OK"
		},
		/* [2] */
		{10, 62, 42, 328},
		StaticText {
			disabled,
			"No editors or translators could be found"
			" to open ^0."
		},
		/* [3] */
		{10, 20, 42, 52},
		Icon {
			disabled,
			0
		},
		/* [4] */
		{49, 266, 77, 332},
		UserItem {
			disabled
		}
	}
};

