/*
	File:		TypeList.cpp

	Contains:	Implementation of class ODTypeList

	Owned by:	Craig Carper

	Copyright:	 1994 - 1996 by Apple Computer, Inc., all rights reserved.

	Change History (most recent first):

		 <2>	 5/24/96	jpa		1246074: SOM_CATCH --> SOM_TRY..SOM_ENDTRY.
									1339104: Use OrderedCollectionIterator in
									Contains method for speed (EditorSetup)

	To Do:
	In Progress:
		
*/

#define VARIABLE_MACROS

#define ODTypeList_Class_Source
#include <TypeList.xih>

#ifndef SOM_ODTypeListIterator_xh
#include <TypLsItr.xh>
#endif

#ifndef SOM_ODObject_xh
#include <ODObject.xh>
#endif

#ifndef _ORDCOLL_
#include "OrdColl.h"
#endif

#ifndef _EXCEPT_
#include "Except.h"			//jpa
#endif

#ifndef _ISOSTR_
#include "ISOStr.h"
#endif

#ifndef _ODMEMORY_
#include "ODMemory.h"
#endif

#ifndef _ODNEW_
#include "ODNew.h"
#endif

#pragma segment ODTypeList

//==============================================================================
// ODTypeList
//==============================================================================

//------------------------------------------------------------------------------
// ODTypeList: somUninit
//------------------------------------------------------------------------------

SOM_Scope void  SOMLINK ODTypeListsomUninit(ODTypeList *somSelf)
{
    ODTypeListData *somThis = ODTypeListGetData(somSelf);
    ODTypeListMethodDebug("ODTypeList","somUninit");

	_fList->DeleteAll();
	ODDeleteObject(_fList);

    parent_somUninit(somSelf);
}

//------------------------------------------------------------------------------
// ODTypeList: InitTypeList
//------------------------------------------------------------------------------

SOM_Scope void  SOMLINK ODTypeListInitTypeList(ODTypeList *somSelf, Environment *ev,
		ODTypeList* typeList)
{
    ODTypeListData *somThis = ODTypeListGetData(somSelf);
    ODTypeListMethodDebug("ODTypeList","InitTypeList");

	ODType type = kODNULL;		ODVolatile(type);
	ODTypeListIterator* iter = kODNULL;	ODVolatile(iter);

	SOM_TRY

		/* Moved from somInit. SOM itself sets fields to zero
		_fList = kODNULL;
		*/
		somSelf->InitObject(ev);
	
		_fList = new OrderedCollection;
	
		if (typeList != (ODTypeList*) kODNULL)
		{
			iter = typeList->CreateTypeListIterator(ev);

			for (type = iter->First(ev);
				 iter->IsNotComplete(ev);
				 type = iter->Next(ev))
			{
				if ( somSelf->Contains(ev,type) == false )
				{
					_fList->AddLast(type);
					type = kODNULL;
				}
				else
					ODDisposePtr(type);
			}
		}
	
	SOM_CATCH_ALL

		ODDisposePtr(type);
		ODDeleteObject(_fList);

	SOM_ENDTRY
	
	ODDeleteObject(iter);
}

//------------------------------------------------------------------------------
// ODTypeList: AddLast
//------------------------------------------------------------------------------

SOM_Scope void  SOMLINK ODTypeListAddLast(ODTypeList *somSelf, Environment *ev,
		ODType type)
{
    ODTypeListData *somThis = ODTypeListGetData(somSelf);
    ODTypeListMethodDebug("ODTypeList","AddLast");

	SOM_TRY

	if ( _fList == kODNULL)
		THROW(kODErrOutOfMemory);

	if ( somSelf->Contains(ev,type) == false )
	{
		ODULong strLength = ODISOStrLength((const ODISOStr)type);
		ElementType newType = ODNewPtrClear(strLength+1, kDefaultHeapID);
		ODISOStrNCopy((const ODISOStr)newType, (ODISOStr)type, strLength );
		_fList->AddLast(newType);
	}

	SOM_CATCH_ALL
	SOM_ENDTRY
}

//------------------------------------------------------------------------------
// ODTypeList: Remove
//------------------------------------------------------------------------------

SOM_Scope void  SOMLINK ODTypeListRemove(ODTypeList *somSelf, Environment *ev,
		ODType type)
{
    ODTypeListData *somThis = ODTypeListGetData(somSelf);
    ODTypeListMethodDebug("ODTypeList","Remove");

	OrderedCollectionIterator* iterator = kODNULL;

	SOM_TRY
		
	if ( type != kODNULL )
	{
		iterator = somSelf->GetImplementation(ev)->CreateIterator();
		THROW_IF_NULL(iterator);
	
		for (ODType thisOne = (ODType) iterator->First(); 
			 iterator->IsNotComplete(); 
			 thisOne = (ODType) iterator->Next())
		{
			if ( thisOne && ODISOStrEqual(thisOne,type) )
			{
				_fList->Remove((ElementType) thisOne);
				break;
			}
		}
	}

	SOM_CATCH_ALL
	
	SOM_ENDTRY

	ODDeleteObject(iterator);
}

//------------------------------------------------------------------------------
// ODTypeList: Contains
//------------------------------------------------------------------------------

SOM_Scope ODBoolean  SOMLINK ODTypeListContains(ODTypeList *somSelf, Environment *ev,
		ODType type)
{
    ODTypeListData *somThis = ODTypeListGetData(somSelf);
    ODTypeListMethodDebug("ODTypeList","Contains");

	ODBoolean result = kODFalse;

	SOM_TRY

	// Use OrderedCollectionIterator for speed:
	OrderedCollectionIterator* iter = _fList->CreateIterator();
	
	for (ODType thisOne = (ODType)iter->First(); 
		 iter->IsNotComplete(); 
		 thisOne = (ODType)iter->Next())
	{
		if ( thisOne )
		{
			if ( (thisOne == type) || ODISOStrEqual(thisOne,type) )
			{
				result = kODTrue;
				break;
			}
		}
	}
	
	delete iter;

	SOM_CATCH_ALL
	SOM_ENDTRY
	return result;
}

//------------------------------------------------------------------------------
// ODTypeList: Count
//------------------------------------------------------------------------------

SOM_Scope ODULong  SOMLINK ODTypeListCount(ODTypeList *somSelf, Environment *ev)
{
    ODTypeListData *somThis = ODTypeListGetData(somSelf);
    ODTypeListMethodDebug("ODTypeList","Count");

	if ( _fList == kODNULL)
	{
		ODSetSOMException(ev, kODErrOutOfMemory);
		return 0;
	}
	else
	{
		return _fList->Count();
	}
}

//------------------------------------------------------------------------------
// ODTypeList: CreateTypeListIterator
//------------------------------------------------------------------------------

SOM_Scope ODTypeListIterator*  SOMLINK ODTypeListCreateTypeListIterator(ODTypeList *somSelf, Environment *ev)
{
    ODTypeListData *somThis = ODTypeListGetData(somSelf);
    ODTypeListMethodDebug("ODTypeList","CreateTypeListIterator");

	ODTypeListIterator* typeListIter = kODNULL; ODVolatile(typeListIter);
	
	SOM_TRY
	
	if ( _fList == kODNULL)
		ODSetSOMException(ev, kODErrOutOfMemory);
	else
	{
		typeListIter = new ODTypeListIterator;
		if (typeListIter == kODNULL)
			ODSetSOMException(ev, kODErrOutOfMemory);
		else
			typeListIter->InitODTypeListIterator(ev, somSelf);
	}
	
	SOM_CATCH_ALL
		ODDeleteObject(typeListIter);
	SOM_ENDTRY
	
	return typeListIter;
}

//------------------------------------------------------------------------------
// ODTypeList: GetImplementation
//------------------------------------------------------------------------------

SOM_Scope OrderedCollection*  SOMLINK ODTypeListGetImplementation(ODTypeList *somSelf, Environment *ev)
{
    ODTypeListData *somThis = ODTypeListGetData(somSelf);
    ODTypeListMethodDebug("ODTypeList","ODTypeListGetImplementation");

	if ( _fList == kODNULL)
		ODSetSOMException(ev, kODErrOutOfMemory);

    return _fList;
}
