/*
	File:		WinUtilM.h

	Contains:	Macintosh window utility functions and classes

	Owned by:	Richard Rodseth

	Copyright:	 1994 - 1995 by Apple Computer, Inc., all rights reserved.

	Change History (most recent first):

		 <5>	 6/28/95	RR		1242642 BB Mostly ref counting. Added
									WindowLink::ShouldRemove
		 <4>	 6/22/95	RR		#1245283 Undoable frame deletion
									#1209427 Changed private api between
									iterator and iteratee. Allow deletion while
									iterating. Added fRemove flag to WindowLink
		 <3>	 5/25/95	jpa		List.h --> LinkList.h. Removed
									GetNextWindow, which is now in Toolbox
									<Windows.h> [1253324, 1241078]
		 <2>	 4/14/95	T		With RR & CG: #1194507 DR/BB:title bar of a
									draft window doesn't reveal it is a draft
									or which draft it is.  Added
									GetDraftNumFromDraft.
		 <1>	 6/16/94	RR		first checked in

	To Do:
	In Progress:
		
*/

#ifndef _WINUTILM_
#define _WINUTILM_

#ifndef _LINKLIST_
#include <LinkList.h>
#endif

#ifndef _ODTYPES_
#include "ODTypes.h"
#endif

//==============================================================================
// Classes used by this interface
//==============================================================================

class ODWindow;

//==============================================================================
// Function Prototypes
//==============================================================================

WindowPtr GetWindowList();
void SetWindowList(WindowPtr theWindow);
//WindowPtr GetNextWindow(WindowPtr theWindow);
void SetNextWindow(WindowPtr theWindow, WindowPtr nextWindow);
ODBoolean GetWindowHilite(WindowPtr theWindow);
void SetWindowHilite(WindowPtr theWindow, ODBoolean windowHilite);
RgnHandle GetStructureRegion(WindowPtr theWindow);
RgnHandle GetContentRegion(WindowPtr theWindow);
ODBoolean	GetWindowVisible(WindowPtr theWindow);
ODULong GetDraftNumFromDraft(Environment* ev, ODDraft* draft);
//=====================================================================================
// WindowLink
//=====================================================================================

class WindowLink : public Link
{
	public:
		WindowLink(ODID windowID, ODWindow* window);	
		~WindowLink() {}
		
		ODBoolean ShouldRemove();
		
		ODID		fID;
		ODWindow* fWindow;
		ODBoolean fRemove;
};

//=====================================================================================
// WindowListIterator - iterates over the Window Manager window list. Iterating from
// back to front using Last/Previous obviously results in several scans of the list,
// as it's a singly-linked list
//=====================================================================================

																		
class WindowListIterator
{
public:
	WindowListIterator();
	~WindowListIterator();

	ODBoolean IsNotComplete();					

	WindowPtr First();

	WindowPtr Next();

	WindowPtr Last();

	WindowPtr Previous();

private:
	WindowPtr fCurrentWindow;
};


#endif //_WINUTILM_